package cz.uhk.pavolpodstreleny.schoolproject.data

import android.arch.lifecycle.LiveData
import android.util.Log
import com.google.firebase.database.*


/**
 * Created by pavol.podstreleny on 06.01.2018.
 */
class FirebaseQueryListLiveData : LiveData<ArrayList<DataSnapshot>> {

    private val TAG = "FirebaseQueryList"
    private var mType: QueryType? = null
    private val mQuery: Query
    private val mValueEventListener: FirebaseValueEventListener

    constructor(query: Query, type: QueryType? = null) {
        mQuery = query
        mType = type
        mValueEventListener = FirebaseValueEventListener()
    }

    constructor(databaseReference: DatabaseReference, type: QueryType? = null) {
        mQuery = databaseReference
        mType = type
        mValueEventListener = FirebaseValueEventListener()
    }

    override fun onActive() {
        super.onActive()
        mQuery.addValueEventListener(mValueEventListener)
    }

    override fun onInactive() {
        super.onInactive()
        mQuery.removeEventListener(mValueEventListener)
    }


    private inner class FirebaseValueEventListener : ValueEventListener {

        override fun onCancelled(error: DatabaseError?) {
            Log.e(TAG, "Problem with loading data from database: onCancelled method called!! message: ${error?.message}")
        }

        override fun onDataChange(data: DataSnapshot?) {
            when (mType) {
                QueryType.ALL_USERS, QueryType.ALL_CATEGORIES, QueryType.ALL_TOPICS -> {
                    if (data != null) {
                        val list = arrayListOf<DataSnapshot>()
                        for (snapshot in data.children) {
                            if (snapshot != null) {
                                list.add(snapshot)
                            }
                        }
                        postValue(list)
                    }

                }

            }
        }
    }
}