package cz.uhk.pavolpodstreleny.schoolproject.data

import android.arch.lifecycle.LiveData
import android.util.Log
import com.google.firebase.database.*

/**
 * Created by pavol.podstreleny on 06.01.2018.
 */
class FirebaseQueryMultipleQueryLiveData : LiveData<List<DataSnapshot>> {

    private val LOG = "MultipleQueryLiveData"
    private var mList = arrayListOf<DataSnapshot>()

    private val mQuery: Query
    private val mChildEventListener: FirebaseChildEventListener


    constructor(query: Query) {
        mQuery = query
        mChildEventListener = FirebaseChildEventListener()
    }

    constructor(databaseReference: DatabaseReference) {
        mQuery = databaseReference
        mChildEventListener = FirebaseChildEventListener()
    }

    override fun onActive() {
        super.onActive()
        mQuery.addChildEventListener(mChildEventListener)
    }

    override fun onInactive() {
        super.onInactive()
        mQuery.removeEventListener(mChildEventListener)
        mList.clear()
    }

    private inner class FirebaseChildEventListener : ChildEventListener {

        override fun onChildAdded(p0: DataSnapshot?, p1: String?) {
            val mChildReference = FirebaseDatabase.getInstance().reference.child("badges").child(p0?.key)
            mChildReference.addListenerForSingleValueEvent(FirebaseValueEventListener())
        }

        override fun onCancelled(p0: DatabaseError?) {
            Log.v(LOG, "ChildEventListener was canceled")
        }

        override fun onChildMoved(p0: DataSnapshot?, p1: String?) {
            Log.v(LOG, "ChildEventListener was moved")
        }

        override fun onChildChanged(p0: DataSnapshot?, p1: String?) {
            Log.v(LOG, "ChildEventListener was changed")
        }

        override fun onChildRemoved(p0: DataSnapshot?) {
            Log.v(LOG, "ChildEventListener was removed")
        }
    }

    private inner class FirebaseValueEventListener : ValueEventListener {
        override fun onCancelled(p0: DatabaseError?) {
            Log.v(LOG, "ValueEventListener was canceled")
        }

        override fun onDataChange(p0: DataSnapshot?) {
            if (p0 != null) {
                mList.add(p0)
            }
            postValue(mList)
        }
    }
}