package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MediatorLiveData
import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseReference
import com.google.firebase.database.FirebaseDatabase
import com.google.firebase.database.Query
import cz.uhk.pavolpodstreleny.schoolproject.data.DatabaseContract
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseQueryListLiveData
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseQueryLiveData
import cz.uhk.pavolpodstreleny.schoolproject.data.QueryType
import cz.uhk.pavolpodstreleny.schoolproject.data.spinner.CategoryListSingleEvent
import cz.uhk.pavolpodstreleny.schoolproject.data.spinner.ExerciseListSingleEvent
import cz.uhk.pavolpodstreleny.schoolproject.data.spinner.GroupListSingleEvent
import cz.uhk.pavolpodstreleny.schoolproject.data.spinner.TopicListSingleEvent
import cz.uhk.pavolpodstreleny.schoolproject.model.*
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.SingleLiveEvent

/**
 * Created by pavol.podstreleny on 13.01.2018.
 */
abstract class BaseRepository {

    protected val mDatabase = FirebaseDatabase.getInstance()
    protected val UNKNOWN = "unknown"

    private var mCategoryReference: DatabaseReference? = null
    private var mTopicReference: Query? = null

    private var mCategoryLiveData: FirebaseQueryListLiveData? = null
    private var mCategorySingleLiveData: CategoryListSingleEvent? = null
    private var mTopicSingleLiveData: TopicListSingleEvent? = null
    private var mMediatorCategoryData: MediatorLiveData<ArrayList<Category>> = MediatorLiveData()


    fun getAllTasksInExercise(id: String?): LiveData<ArrayList<Task>>? {
        val mediatorLiveData = MediatorLiveData<ArrayList<Task>>()
        val reference = mDatabase.reference.child(DatabaseContract.TABLE_TASKS_NAME).orderByChild(DatabaseContract.TABLE_TASKS_EXERCISE).equalTo(id)
        val data = FirebaseQueryListLiveData(reference, QueryType.ALL_TOPICS)
        transformTaskData(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformTaskData(mediatorLiveData: MediatorLiveData<ArrayList<Task>>, data: FirebaseQueryListLiveData) {
        mediatorLiveData.addSource(data, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<Task>()
                for (datas in snapshot) {

                    val task = Task()
                    task.description = datas.child(DatabaseContract.TABLE_EXERCISES_DESCRIPTION).getValue(String::class.java)
                    task.exercise = datas.child(DatabaseContract.TABLE_EXERCISES_NAME).getValue(String::class.java)
                    task.photoURL = datas.child(DatabaseContract.TABLE_TASKS_PHOTOURL).getValue(String::class.java)

                    datas.child("choices").children.forEach { snapshot ->
                        val boolValue = snapshot.value
                        if (boolValue is Boolean) {
                            task.choices?.put(snapshot.key.toString(), boolValue)
                        }
                    }
                    task.taskID = datas.key
                    list.add(task)
                }
                mediatorLiveData.postValue(list)
            }
        })
    }


    fun getUserPoints(userID: String): LiveData<Double> {
        val mediatorLiveData = MediatorLiveData<Double>()
        val query = mDatabase.reference.child(DatabaseContract.TABLE_USERS_NAME).child(userID).child(DatabaseContract.TABLE_USERS_SCORE)
        val data = FirebaseQueryLiveData(query)
        transformPoints(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformPoints(mediatorLiveData: MediatorLiveData<Double>, data: FirebaseQueryLiveData) {
        mediatorLiveData.addSource(data, { t: DataSnapshot? ->
            if (t != null) {
                val points = t.getValue(Double::class.java)
                mediatorLiveData.postValue(points)
            }
        })
    }


    fun getAllExercisesInTopic(id: String): LiveData<ArrayList<Exercise>> {
        val mediatorLiveData = MediatorLiveData<ArrayList<Exercise>>()
        val reference = mDatabase.reference.child(DatabaseContract.TABLE_EXERCISES_NAME).orderByChild(DatabaseContract.TABLE_EXERCISES_GROUP).equalTo(id)
        val data = FirebaseQueryListLiveData(reference, QueryType.ALL_TOPICS)
        transformExerciseData(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformExerciseData(mediatorLiveData: MediatorLiveData<ArrayList<Exercise>>, data: FirebaseQueryListLiveData) {
        mediatorLiveData.addSource(data, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<Exercise>()
                for (data in snapshot) {
                    val exercise = data.getValue(Exercise::class.java)
                    exercise?.exerciseKey = data.key
                    if (data.hasChild("tasks")) {
                        data.child("tasks").children.forEach { datasnapshot ->
                            exercise?.taskMap?.set(datasnapshot.key, datasnapshot.value)
                        }
                    }
                    list.add(exercise!!)
                }
                mediatorLiveData.postValue(list)
            }
        })
    }


    fun getAllExercisesInGroupSpinner(id: String): SingleLiveEvent<List<Exercise>> {
        val mExerciseRerefence = mDatabase.reference.child(DatabaseContract.TABLE_EXERCISES_NAME).orderByChild(DatabaseContract.TABLE_EXERCISES_GROUP).equalTo(id)
        val mExerciseSingleLiveData = ExerciseListSingleEvent(mExerciseRerefence)
        return mExerciseSingleLiveData
    }

    fun getAllTopicsInCategorySpinner(id: String): SingleLiveEvent<List<Topic>> {
        mTopicReference = mDatabase.reference.child(DatabaseContract.TABLE_TOPICS_NAME).orderByChild(DatabaseContract.TABLE_TOPICS_CATEGORY).equalTo(id)
        mTopicSingleLiveData = TopicListSingleEvent(mTopicReference!!)
        return mTopicSingleLiveData!!
    }

    fun getAllGroupsInTopicSpinner(id: String): SingleLiveEvent<List<Group>> {
        val reference = mDatabase.reference.child(DatabaseContract.TABLE_GROUPS_NAME).orderByChild(DatabaseContract.TABLE_GROUPS_TOPIC).equalTo(id)
        val data = GroupListSingleEvent(reference)
        return data
    }

    fun getAllTopicsInCategory(id: String): LiveData<ArrayList<Topic>> {
        val mediatorLiveData = MediatorLiveData<ArrayList<Topic>>()
        val reference = mDatabase.reference.child(DatabaseContract.TABLE_TOPICS_NAME).orderByChild(DatabaseContract.TABLE_TOPICS_CATEGORY).equalTo(id)
        val data = FirebaseQueryListLiveData(reference, QueryType.ALL_TOPICS)
        transformTopicData(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformTopicData(mediatorLiveData: MediatorLiveData<ArrayList<Topic>>, data: LiveData<ArrayList<DataSnapshot>>) {
        mediatorLiveData.addSource(data, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<Topic>()
                for (data in snapshot) {
                    val topic = data.getValue(Topic::class.java)
                    topic?.topicKey = data.key
                    list.add(topic!!)
                }
                mediatorLiveData.postValue(list)
            }
        })
    }


    fun getAllGroupsInTopic(id: String): LiveData<ArrayList<Group>> {
        val mediatorLiveData = MediatorLiveData<ArrayList<Group>>()
        val reference = mDatabase.reference.child(DatabaseContract.TABLE_GROUPS_NAME).orderByChild(DatabaseContract.TABLE_GROUPS_TOPIC).equalTo(id)
        val data = FirebaseQueryListLiveData(reference, QueryType.ALL_TOPICS)
        transformGroupData(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformGroupData(mediatorLiveData: MediatorLiveData<ArrayList<Group>>, data: LiveData<ArrayList<DataSnapshot>>) {
        mediatorLiveData.addSource(data, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<Group>()
                for (data in snapshot) {
                    val group = data.getValue(Group::class.java)
                    group?.groupKey = data.key
                    list.add(group!!)
                }
                mediatorLiveData.postValue(list)

            }
        })
    }


    fun getAllCategoriesForSpinner(): SingleLiveEvent<List<Category>> {
        if (mCategoryReference == null) {
            mCategoryReference = mDatabase.reference.child(DatabaseContract.TABLE_CATEGORY_NAME)
            mCategorySingleLiveData = CategoryListSingleEvent(mCategoryReference!!)
        }
        return mCategorySingleLiveData!!
    }


    fun getAllCategories(): LiveData<ArrayList<Category>> {

        if (mCategoryReference == null) {
            mCategoryReference = mDatabase.reference.child(DatabaseContract.TABLE_CATEGORY_NAME)
            mCategoryLiveData = FirebaseQueryListLiveData(mCategoryReference!!, QueryType.ALL_CATEGORIES)
            transformCategoriesData()
        }

        return mMediatorCategoryData
    }


    private fun transformCategoriesData() {
        mMediatorCategoryData.addSource(mCategoryLiveData!!, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<Category>()
                for (date in snapshot) {
                    val category = date.getValue(Category::class.java)
                    category?.categoryKey = date.key
                    if (date.key != UNKNOWN) {
                        list.add(category!!)
                    }
                }
                mMediatorCategoryData.postValue(list)

            }
        })
    }


}