package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.arch.lifecycle.LiveData
import android.net.Uri
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.DatabaseReference
import com.google.firebase.storage.FirebaseStorage
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.DatabaseContract
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseAdminLiveData
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult
import cz.uhk.pavolpodstreleny.schoolproject.data.await

/**
 * Created by pavol.podstreleny on 07.01.2018.
 */
class CategoryRepository : BaseRepository() {

    private val TAG = "CategoryRepository"

    private val mFirebaseStorage = FirebaseStorage.getInstance()
    private val mCategoryFileReference = mFirebaseStorage.reference.child(DatabaseContract.FILE_CATEGORY_PATH)

    private var mAdminReference: DatabaseReference? = null
    private var mAdminLiveData: LiveData<Boolean>? = null


    fun isUserAdmin(): LiveData<Boolean>? {
        if (mAdminReference == null) {
            val mUserID = FirebaseAuth.getInstance().currentUser?.uid
            if (mUserID != null) {
                mAdminReference = mDatabase.reference.child(DatabaseContract.TABLE_ADMINS_NAME).child(mUserID)
                mAdminLiveData = FirebaseAdminLiveData(mAdminReference!!)
            }
        }
        return mAdminLiveData
    }


    suspend fun addCategory(uriImagePath: Uri, categoryName: String): FirebaseResult {
        val categoryImage = addCategoryImage(uriImagePath)
        return if (categoryImage != null) {
            if (addCategoryData(categoryImage, categoryName)) {
                FirebaseResult(true, R.string.category_admin_page_category_added)
            } else {
                if (deleteCategoryImage(uriImagePath.lastPathSegment)) {
                    FirebaseResult(false, R.string.category_admin_page_category_added_error)
                } else {
                    FirebaseResult(false, R.string.category_admin_page_category_added_error)
                }
            }
        } else {
            FirebaseResult(false, R.string.category_admin_page_category_added_error)
        }
    }

    suspend private fun addCategoryImage(uri: Uri): Uri? {

        val photoRef = mCategoryFileReference.child(uri.lastPathSegment + Math.random())

        try {
            val saveFile = photoRef.putFile(uri).await()
            return saveFile.downloadUrl
        } catch (e: Exception) {
            return null
        }
    }

    suspend private fun addCategoryData(downloadUri: Uri?, categoryName: String): Boolean {

        val map = hashMapOf<String, Any>()
        map.put(DatabaseContract.TABLE_CATEGORY_DESCRIPTION, categoryName)
        map.put(DatabaseContract.TABLE_CATEGORY_PHOTOURL, downloadUri.toString())

        val reference = mDatabase.reference.child(DatabaseContract.TABLE_CATEGORY_NAME)

        try {
            reference.push().setValue(map)
            return true
        } catch (e: Exception) {
            return false
        }

    }


    suspend fun deleteCategory(urlCategory: String, imageName: String): FirebaseResult {
        if (urlCategory == UNKNOWN) {
            return FirebaseResult(false, R.string.category_admin_page_category_unknown_delete_restriction)
        } else {
            if (deleteCategoryImage(imageName)) {
                if (deleteCategoryData(urlCategory)) {
                    return FirebaseResult(true, R.string.category_admin_page_category_deleted)
                } else {
                    return FirebaseResult(false, R.string.category_admin_page_category_deleted_problem)
                }
            } else {
                return FirebaseResult(false, R.string.category_admin_page_category_deleted_problem)
            }
        }

    }


    suspend private fun deleteCategoryImage(imageName: String): Boolean {
        val photoRef = mFirebaseStorage.reference.child(imageName)
        try {
            photoRef.delete().await()
            return true
        } catch (e: Exception) {
            return false
        }
    }

    suspend private fun deleteCategoryData(url: String): Boolean {
        val reference = mDatabase.reference.child(DatabaseContract.TABLE_CATEGORY_NAME).child(url)
        try {
            reference.removeValue().await()
            return true
        } catch (e: Exception) {
            return false
        }
    }


    suspend fun editCategory(isImageChanged: Boolean, imageUri: Uri?, oldUri: String, categoryName: String, categoryPath: String): FirebaseResult {
        val uri = Uri.parse(oldUri)
        if (isImageChanged && imageUri != null) {
            val categoryImage = addCategoryImage(imageUri)
            if (categoryImage != null) {
                if (editCategoryData(categoryName, categoryImage.toString(), categoryPath)) {
                    if (deleteCategoryImage(uri.lastPathSegment)) {
                        return FirebaseResult(true, R.string.category_admin_page_category_edit)
                    } else {
                        return FirebaseResult(true, R.string.category_admin_page_category_edit_problem)
                    }
                } else {
                    return FirebaseResult(false, R.string.category_admin_page_category_edit_problem)
                }
            } else {
                return FirebaseResult(true, R.string.category_admin_page_category_edit_problem)
            }
        } else {
            if (editCategoryData(categoryName, null, categoryPath)) {
                return FirebaseResult(true, R.string.category_admin_page_category_edit)
            } else {
                return FirebaseResult(false, R.string.category_admin_page_category_edit_problem)
            }
        }

    }


    suspend private fun editCategoryData(categoryName: String, imagePath: String?, categoryPath: String): Boolean {
        val map = hashMapOf<String, Any>()
        map.put(DatabaseContract.TABLE_CATEGORY_DESCRIPTION, categoryName)
        if (imagePath != null) {
            map.put(DatabaseContract.TABLE_CATEGORY_PHOTOURL, imagePath)
        }

        val reference = mDatabase.reference.child(DatabaseContract.TABLE_CATEGORY_NAME).child(categoryPath)

        try {
            reference.updateChildren(map)
            return true
        } catch (e: Exception) {
            return false
        }

    }


}