package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MediatorLiveData
import com.google.firebase.database.DatabaseReference
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.DatabaseContract
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseQueryLiveData
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult
import cz.uhk.pavolpodstreleny.schoolproject.model.UserProfile
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 15.01.2018.
 */
class ExerciseAdminRepository : AdminRepository() {

    private var mUserLiveData: FirebaseQueryLiveData? = null
    private var mUserReference: DatabaseReference? = null
    private var mMediatorUserData: MediatorLiveData<UserProfile> = MediatorLiveData()

    suspend fun deleteExercise(groupID: String): FirebaseResult {
        return if (deleteData(DatabaseContract.TABLE_EXERCISES_NAME, groupID)) {
            FirebaseResult(true, R.string.exercise_admin_page_exercise_deleted)
        } else {
            FirebaseResult(false, R.string.exercise_admin_page_exercise_deleted_error)
        }
    }

    fun getUser(childUID: String): LiveData<UserProfile> {
        if (mUserReference == null) {
            mUserReference = mDatabase.reference.child(DatabaseContract.TABLE_USERS_NAME).child(childUID)
            mUserLiveData = FirebaseQueryLiveData(mUserReference!!)
            transformUserData()
        }
        return mMediatorUserData
    }

    suspend fun addExercise(describtion: String, groupID: String?): FirebaseResult {
        if (groupID == null) return FirebaseResult(false, R.string.exercise_admin_page_exercise_deleted_error)

        val map = HashMap<String, Any>()
        map.put(DatabaseContract.TABLE_EXERCISES_DESCRIPTION, describtion)
        map.put(DatabaseContract.TABLE_EXERCISES_GROUP, groupID)

        return if (addData(DatabaseContract.TABLE_EXERCISES_NAME, map)) {
            FirebaseResult(true, R.string.exercise_admin_page_exercise_added)
        } else {
            FirebaseResult(false, R.string.exercise_admin_page_exercise_added_error)
        }
    }

    suspend fun editExercise(groupID: String?, exerciseIDNumber: String?, description: String): FirebaseResult {
        if (groupID == null || exerciseIDNumber == null) {
            return FirebaseResult(false, R.string.exercise_admin_page_exercise_updated_error)
        }
        val map = HashMap<String, Any>()
        map.put(DatabaseContract.TABLE_EXERCISES_GROUP, groupID)
        map.put(DatabaseContract.TABLE_EXERCISES_DESCRIPTION, description)
        if (editData(DatabaseContract.TABLE_EXERCISES_NAME, exerciseIDNumber, map)) {
            return FirebaseResult(true, R.string.exercise_admin_page_exercise_updated)
        } else {
            return FirebaseResult(false, R.string.exercise_admin_page_exercise_updated_error)
        }


    }


    private fun transformUserData() {
        mMediatorUserData.addSource(mUserLiveData!!, { snapshot ->
            if (snapshot != null) {
                launch {
                    val user = UserProfile()
                    user.name = snapshot.child(DatabaseContract.TABLE_USERS_USER_NAME).getValue(String::class.java)
                    user.email = snapshot.child(DatabaseContract.TABLE_USERS_USER_EMAIL).getValue(String::class.java)
                    if (snapshot.child(DatabaseContract.TABLE_USERS_SCORE).getValue(Int::class.java) != null) {
                        user.score = snapshot.child(DatabaseContract.TABLE_USERS_SCORE).getValue(Int::class.java)!!
                    }
                    if (snapshot.child(DatabaseContract.TABLE_USERS_LAST_TASK).getValue(Long::class.java) != null) {
                        user.lastTask = snapshot.child(DatabaseContract.TABLE_USERS_LAST_TASK).getValue(Long::class.java)!!
                    }


                    val mapHash = HashMap<String, List<HashMap<String, Long>>>()


                    val tasks = ArrayList<HashMap<String, Long>>()

                    if (snapshot.child(DatabaseContract.TABLE_EXERCISES_NAME) != null && snapshot.child(DatabaseContract.TABLE_EXERCISES_NAME).childrenCount > 0) {
                        snapshot.child(DatabaseContract.TABLE_EXERCISES_NAME).children.forEach { firstLevel ->

                            val fistKey = firstLevel.key
                            val lister = ArrayList<HashMap<String, Long>>()

                            firstLevel?.children?.forEach { secondLevel ->
                                if (secondLevel != null) {
                                    val map = HashMap<String, Long>()
                                    map.put(secondLevel.key!!, secondLevel.getValue(Long::class.java)!!)
                                    tasks.add(map)
                                    lister.add(map)
                                }
                            }

                            mapHash.put(fistKey, lister)

                        }
                    }

                    if (tasks.size > 0) {
                        user.tasks = tasks
                    }

                    if (mapHash.size > 0) {
                        user.exercisess = mapHash
                    }



                    mMediatorUserData.postValue(user)
                }
            } else {
                mMediatorUserData.value = null
            }
        })
    }


}