package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.net.Uri
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.DatabaseContract
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult

/**
 * Created by pavol.podstreleny on 10.01.2018.
 */
class TopicAdminRepository : AdminRepository() {


    suspend fun deleteTopic(topicID: String, photoURL: Uri): FirebaseResult {
        return if (deleteImage(photoURL.lastPathSegment)) {
            if (deleteData(DatabaseContract.TABLE_TOPICS_NAME, topicID)) {
                FirebaseResult(true, R.string.topic_admin_page_topic_deleted)
            } else {
                FirebaseResult(false, R.string.topic_admin_page_topic_deleted_data_error)
            }
        } else {
            FirebaseResult(false, R.string.topic_admin_page_topic_deleted_image_error)
        }
    }


    suspend fun addTopic(description: String, photoURL: Uri?, categoryID: String?): FirebaseResult {
        if (photoURL == null || categoryID == null) return FirebaseResult(false, R.string.topic_admin_page_topic_added_error)

        val imageAddedUrl = addImage(DatabaseContract.FILE_TOPIC_PATH, photoURL)
        return if (imageAddedUrl != null) {

            val map = hashMapOf<String, Any>()
            map.put(DatabaseContract.TABLE_TOPIC_DESCRIPTION, description)
            map.put(DatabaseContract.TABLE_TOPIC_PHOTO_URL, imageAddedUrl.toString())
            map.put(DatabaseContract.TABLE_TOPICS_CATEGORY, categoryID)

            if (addData(DatabaseContract.TABLE_TOPICS_NAME, map)) {
                showTopicAddSuccess()
            } else {
                showTopicAddError()
            }
        } else {
            showTopicAddError()
        }
    }

    suspend fun editTopic(newImageUrl: Uri?, oldImageUri: Uri?, categoryID: String?, topicID: String?, description: String): FirebaseResult {
        if (categoryID == null || topicID == null || oldImageUri == null) return FirebaseResult(false, R.string.topic_admin_page_topic_updated_error)

        val map = HashMap<String, Any>()
        map.put(DatabaseContract.TABLE_TOPIC_DESCRIPTION, description)
        map.put(DatabaseContract.TABLE_TOPICS_CATEGORY, categoryID)


        val skuska = newImageUrl

        return if (newImageUrl != null) {
            val addImageUri = addImage(DatabaseContract.FILE_TOPIC_PATH, newImageUrl)
            if (addImageUri != null) {

                map.put(DatabaseContract.TABLE_TOPIC_PHOTO_URL, addImageUri.toString())
                if (editData(DatabaseContract.TABLE_TOPICS_NAME, topicID.toString(), map)) {
                    if (deleteImage(oldImageUri.lastPathSegment)) {
                        FirebaseResult(true, R.string.topic_admin_page_topic_updated)
                    } else {
                        FirebaseResult(false, R.string.topic_admin_page_topic_delete_old_image_error)
                    }
                } else {
                    if (deleteImage(addImageUri.lastPathSegment)) {
                        FirebaseResult(false, R.string.topic_admin_page_topic_updated_error)
                    } else {
                        FirebaseResult(false, R.string.topic_admin_page_topic_updated_date_add_error)
                    }
                }
            } else {
                FirebaseResult(false, R.string.topic_admin_page_topic_updated_image_add_error)
            }
        } else {
            if (editData(DatabaseContract.TABLE_TOPICS_NAME, topicID.toString(), map)) {
                FirebaseResult(true, R.string.topic_admin_page_topic_updated)
            } else {
                FirebaseResult(false, R.string.topic_admin_page_topic_updated_error)
            }
        }
    }


    private fun showTopicAddError(): FirebaseResult {
        return FirebaseResult(false, R.string.topic_admin_page_topic_added_error)
    }

    private fun showTopicAddSuccess(): FirebaseResult {
        return FirebaseResult(true, R.string.topic_admin_page_topic_added)
    }

}








