package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MediatorLiveData
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.DatabaseReference
import com.google.firebase.database.Query
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.*
import cz.uhk.pavolpodstreleny.schoolproject.model.Badge
import cz.uhk.pavolpodstreleny.schoolproject.model.UserProfile
import javax.inject.Singleton

/**
 * Created by pavol.podstreleny on 03.01.2018.
 */

@Singleton
class UserRepository() : AdminRepository() {

    private val SHOW_TOP_LEADERS = 10
    private var mUserReference: DatabaseReference? = null
    private var mUserBadgesReference: DatabaseReference? = null
    private var mAllUsersReference: Query? = null


    private var mUserLiveData: FirebaseQueryLiveData? = null
    private var mBadgeLiveData: FirebaseQueryMultipleQueryLiveData? = null
    private var mAllUsersLiveData: FirebaseQueryListLiveData? = null

    private var mMediatorUserData: MediatorLiveData<UserProfile> = MediatorLiveData()
    private var mMediatorUserBadgesData: MediatorLiveData<ArrayList<Badge>> = MediatorLiveData()
    private var mMediatorAllUsersData: MediatorLiveData<ArrayList<UserProfile>> = MediatorLiveData()


    fun getAllUsers(): LiveData<ArrayList<UserProfile>> {
        if (mAllUsersReference == null) {
            mAllUsersReference = mDatabase.reference.child(DatabaseContract.TABLE_USERS_NAME).limitToLast(SHOW_TOP_LEADERS).orderByChild(DatabaseContract.TABLE_USERS_SCORE)
            mAllUsersLiveData = FirebaseQueryListLiveData(mAllUsersReference!!, QueryType.ALL_USERS)
            transformAllUsersData()
        }
        return mMediatorAllUsersData
    }

    suspend fun changeUserName(newUserName: String): FirebaseResult {
        val actualUserID = FirebaseAuth.getInstance().currentUser?.uid
        if (actualUserID != null) {
            val hashMap = HashMap<String, Any>()
            hashMap.put(DatabaseContract.TABLE_USERS_USER_NAME, newUserName)
            if (editData(DatabaseContract.TABLE_USERS_NAME, actualUserID, hashMap)) {
                return FirebaseResult(true, R.string.userProfile_page_user_name_change_success)
            } else {
                return FirebaseResult(false, R.string.userProfile_page_user_name_change_problem)
            }
        } else {
            return FirebaseResult(false, R.string.userProfile_page_user_name_change_problem)
        }
    }

    fun getUser(childUID: String): LiveData<UserProfile> {
        if (mUserReference == null) {
            mUserReference = mDatabase.reference.child(DatabaseContract.TABLE_USERS_NAME).child(childUID)
            mUserLiveData = FirebaseQueryLiveData(mUserReference!!)
            transformUserData()
        }
        return mMediatorUserData
    }

    fun getBadges(childUID: String): LiveData<ArrayList<Badge>> {
        mUserBadgesReference = mDatabase.reference.child(DatabaseContract.TABLE_USER_BADGES_NAME).child(childUID)
        mBadgeLiveData = FirebaseQueryMultipleQueryLiveData(mUserBadgesReference!!)
        transformUserBadgesData()
        return mMediatorUserBadgesData
    }

    private fun transformUserBadgesData() {
        mMediatorUserBadgesData.addSource(mBadgeLiveData!!, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<Badge>()

                for (i in 0..snapshot.size - 1) {
                    if (snapshot.get(i) != null) {
                        list.add(snapshot.get(i).getValue(Badge::class.java)!!)
                    }
                }
                mMediatorUserBadgesData.postValue(list)

            }
        })
    }

    private fun transformAllUsersData() {
        mMediatorAllUsersData.addSource(mAllUsersLiveData!!, { snapshot ->
            if (snapshot != null) {

                val list = arrayListOf<UserProfile>()

                for (date in snapshot) {

                    val user = UserProfile()

                    user.name = date.child(DatabaseContract.TABLE_USERS_USER_NAME).getValue(String::class.java)
                    user.email = date.child(DatabaseContract.TABLE_USERS_USER_EMAIL).getValue(String::class.java)
                    val scoreValue = date.child(DatabaseContract.TABLE_USERS_SCORE).getValue(Int::class.java)
                    if (scoreValue != null) {
                        user.score = scoreValue
                    }
                    user.uid = date.key

                    list.add(user)
                }

                mMediatorAllUsersData.postValue(list)

            }
        })
    }

    private fun transformUserData() {
        mMediatorUserData.addSource(mUserLiveData!!, { snapshot ->
            if (snapshot != null) {

                val user = UserProfile()
                user.name = snapshot.child(DatabaseContract.TABLE_USERS_USER_NAME).getValue(String::class.java)
                user.email = snapshot.child(DatabaseContract.TABLE_USERS_USER_EMAIL).getValue(String::class.java)
                if (snapshot.child(DatabaseContract.TABLE_USERS_SCORE).getValue(Int::class.java) != null) {
                    user.score = snapshot.child(DatabaseContract.TABLE_USERS_SCORE).getValue(Int::class.java)!!
                }
                if (snapshot.child(DatabaseContract.TABLE_USERS_LAST_TASK).getValue(Long::class.java) != null) {
                    user.lastTask = snapshot.child(DatabaseContract.TABLE_USERS_LAST_TASK).getValue(Long::class.java)!!
                }


                val mapHash = HashMap<String, List<HashMap<String, Long>>>()


                val tasks = ArrayList<HashMap<String, Long>>()

                if (snapshot.child(DatabaseContract.TABLE_EXERCISES_NAME) != null && snapshot.child(DatabaseContract.TABLE_EXERCISES_NAME).childrenCount > 0) {
                    snapshot.child(DatabaseContract.TABLE_EXERCISES_NAME).children.forEach { firstLevel ->

                        val fistKey = firstLevel.key
                        val lister = ArrayList<HashMap<String, kotlin.Long>>()

                        firstLevel?.children?.forEach { secondLevel ->
                            if (secondLevel != null) {
                                val map = HashMap<String, Long>()
                                map.put(secondLevel.key!!, secondLevel.getValue(Long::class.java)!!)
                                tasks.add(map)
                                lister.add(map)
                            }
                        }

                        mapHash.put(fistKey, lister)

                    }
                }

                if (tasks.size > 0) {
                    user.tasks = tasks
                }

                if (mapHash.size > 0) {
                    user.exercisess = mapHash
                }



                mMediatorUserData.postValue(user)

            } else {
                mMediatorUserData.value = null
            }
        })
    }

}