package cz.uhk.pavolpodstreleny.schoolproject.data.spinner

import android.util.Log
import com.google.firebase.database.*
import cz.uhk.pavolpodstreleny.schoolproject.model.Category
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.SingleLiveEvent


/**
 * Created by pavol.podstreleny on 31.01.2018.
 */
class CategoryListSingleEvent : SingleLiveEvent<List<Category>> {

    private val mQuery: Query
    private val mValueEventListener: FirebaseValueEventListener
    private val TAG = "CategoryListSingleEvent"

    constructor(query: Query) {
        mQuery = query
        mValueEventListener = FirebaseValueEventListener()
    }

    constructor(databaseReference: DatabaseReference) {
        mQuery = databaseReference
        mValueEventListener = FirebaseValueEventListener()
    }

    override fun onActive() {
        super.onActive()
        mQuery.addValueEventListener(mValueEventListener)
    }

    override fun onInactive() {
        super.onInactive()
        mQuery.removeEventListener(mValueEventListener)
    }

    private inner class FirebaseValueEventListener : ValueEventListener {

        override fun onCancelled(error: DatabaseError?) {
            Log.e(TAG, "Problem with loading data from database: onCancelled method called!! message: ${error?.message}")
        }

        override fun onDataChange(data: DataSnapshot?) {

            if (data != null) {
                val list = arrayListOf<Category>()
                for (snapshot in data.children) {
                    if (snapshot != null) {
                        val category = snapshot.getValue(Category::class.java)
                        category!!.categoryKey = snapshot.key
                        list.add(category)
                    }
                }
                postValue(list)
            }

        }
    }


}