package cz.uhk.pavolpodstreleny.schoolproject.screens

import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v7.app.AppCompatActivity
import android.view.View
import android.widget.Toast
import com.firebase.ui.auth.AuthUI
import com.firebase.ui.auth.ErrorCodes
import com.firebase.ui.auth.IdpResponse
import com.google.firebase.auth.FirebaseAuth
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.explore.ExploreFragment
import cz.uhk.pavolpodstreleny.schoolproject.screens.leaderboard.LeaderboardFragment
import cz.uhk.pavolpodstreleny.schoolproject.screens.user_profile.UserProfileFragment
import kotlinx.android.synthetic.main.activity_dashboard.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network.*
import kotlinx.android.synthetic.main.view_no_network_connectino_small.*

class MainActivity : AppCompatActivity(), FirebaseAuth.AuthStateListener, INetworkChecker {

    private var mFragment: Fragment? = null
    private var mFragmentManager: FragmentManager? = null
    private var mAuth = FirebaseAuth.getInstance()

    private val TITLE_KEY = "TITLE_KEY"

    private val RC_SIGN = 2
    private val RC_LOG_OUT = 1

    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_dashboard)


        setSupportActionBar(toolbar)

        //ConnectionReceiver
        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet),this,this)

        if(savedInstanceState != null && savedInstanceState.containsKey(TITLE_KEY)){
            supportActionBar?.title = savedInstanceState.getString(TITLE_KEY)
        }else{
            setUpExploreFragment()
        }

        no_internet_connection_button.setOnClickListener{
            startDifferentActivity()
        }

        bottomNavigation.setOnNavigationItemSelectedListener { item ->
            when (item.itemId) {
                R.id.action_explore -> setUpExploreFragment()
                R.id.action_leader -> setUpLeaderboardFragment()
                R.id.action_profile -> setUpProfileFragment()
                else -> false
            }
        }
    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    override fun onAuthStateChanged(p0: FirebaseAuth) {
        if(p0.currentUser == null){
            startDifferentActivity()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == RC_SIGN || requestCode == RC_LOG_OUT) {

            val response = IdpResponse.fromResultIntent(data)

            // Successfully signed in
            if (resultCode == RESULT_OK) {
                when(requestCode){
                    RC_SIGN -> {
                        hideErroView()
                        setUpExploreFragment()
                    }
                    RC_LOG_OUT -> {
                        hideErroView()
                        setUpProfileFragment()
                    }
                }
                return
            } else {
                if (response?.getErrorCode() == ErrorCodes.NO_NETWORK) {
                    showNoNetworkError()
                    return
                }

                if (response?.getErrorCode() == ErrorCodes.UNKNOWN_ERROR) {
                    Toast.makeText(this,"Unknown Error",Toast.LENGTH_SHORT).show()
                    return
                }
            }
        }
    }

    override fun onStart() {
        super.onStart()
        mAuth.addAuthStateListener(this)
    }

    override fun onStop() {
        super.onStop()
        mAuth.removeAuthStateListener(this)
    }

    private fun startLogInActivity(sign: Int){
        startActivityForResult(
                AuthUI.getInstance().createSignInIntentBuilder().setIsSmartLockEnabled(false).setAvailableProviders(
                        arrayListOf(AuthUI.IdpConfig.Builder(AuthUI.EMAIL_PROVIDER).build())
                ).build(),sign)
    }

    private fun hideErroView(){
        mainView.visibility = View.VISIBLE
        constraintNoNetwork.visibility = View.GONE
    }

    private fun showNoNetworkError(){
        mainView.visibility = View.GONE
        constraintNoNetwork.visibility = View.VISIBLE
    }

    private fun setUpProfileFragment(): Boolean {
        mFragment = UserProfileFragment()
        setNavigationTitle(R.string.profile_title_text)
        showFragment(mFragment!!)
        return true
    }

    private fun setUpLeaderboardFragment(): Boolean {
        mFragment = LeaderboardFragment()
        setNavigationTitle(R.string.leaderboard_title_text)
        showFragment(mFragment!!)
        return true
    }

    private fun setUpExploreFragment(): Boolean {
        mFragment = ExploreFragment()
        setNavigationTitle(R.string.explore_title_text)
        showFragment(mFragment!!)
        return true
    }

    private fun setNavigationTitle(resource: Int) {
        supportActionBar?.title = getString(resource)
    }

    private fun showFragment(fragment: Fragment) {
        mFragmentManager = supportFragmentManager
        mFragmentManager?.beginTransaction()?.replace(R.id.pager, fragment)?.commit()
    }

    private fun startDifferentActivity(){
        val menu = bottomNavigation.menu
        val menuItem = menu.getItem(0)
        if(menuItem.isChecked){
            startLogInActivity(RC_SIGN)
        }else{
            startLogInActivity(RC_LOG_OUT)
        }
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        outState?.putCharSequence(TITLE_KEY,toolbar.title)
    }
}
