package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.category

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.Action
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Category
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.category.category_add_edit.CategoryEditActivity
import cz.uhk.pavolpodstreleny.schoolproject.screens.explore.CategoryViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_category.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_data.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch


/**
 * Created by pavol.podstreleny on 07.01.2018.
 */

class CategoryAdminActivity : AppCompatActivity(),
        CategoryAdminAdapter.OnCategoryClickListener, INetworkChecker,
        CategoryAdminDeleteDialog.OnDialogClick {

    private val CATEGORY_DIALOG = "CATEGORY_DIALOG"
    private var mCategoryUrl: String? = null
    private var mImageName: String? = null
    private val UNKNOWN = "unknown"

    private var viewModel: CategoryViewModel? = null
    override var connectionObserver: ConnectionObserver? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_category)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.category_admin_page_category_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)


        val mAdapter = CategoryAdminAdapter(this)
        categoryAdminRecyclerView.adapter = mAdapter
        categoryAdminRecyclerView.layoutManager = LinearLayoutManager(this)

        viewModel = ViewModelProviders.of(this).get(CategoryViewModel::class.java)

        viewModel?.getAllSpinnerCategories()?.observe(this, Observer { data ->
            updateCategoryUI(mAdapter, data as ArrayList<Category>)
        })

        addCategory.setOnClickListener {
            showAddCategoryPage()
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun showAddCategoryPage() {
        if (!NetworkUtil.isNetworkAvailable(this)) {
            Snacker.show(mainLayout, getString(R.string.category_admin_page_add_problem))
        } else {
            val intent = Intent(this, CategoryEditActivity::class.java)
            intent.putExtra(ExtraIntents.CATEGORY_ADD, true)
            startActivityForResult(intent, ExtraIntents.CATEGORY_MESSAGE_CODE)
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ExtraIntents.CATEGORY_MESSAGE_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                Snacker.show(mainLayout, data?.getStringExtra(ExtraIntents.CATEGORY_MESSAGE).toString())
            }
        }
    }

    override fun onClick(categoryKey: String, uri: Uri, action: Action, categoryName: String) {
        when (action) {
            Action.DELETE -> showDialog(categoryKey, uri.lastPathSegment)
            Action.EDIT -> startEditCategory(categoryKey, uri.toString(), categoryName)
        }
    }

    private fun updateCategoryUI(adapter: CategoryAdminAdapter, data: ArrayList<Category>?) {
        if (data != null) {
            loaderProgress.visibility = View.GONE
            adapter.swapData(data)
            if (data.size != 0) {
                noAvailableData.visibility = View.GONE
                categoryAdminRecyclerView.visibility = View.VISIBLE
            } else {
                noAvailableData.visibility = View.VISIBLE
                categoryAdminRecyclerView.visibility = View.GONE
            }
        }
    }

    private fun startEditCategory(url: String, uri: String, categoryName: String) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (url != UNKNOWN) {
                val intent = Intent(this, CategoryEditActivity::class.java)
                intent.putExtra(ExtraIntents.CATEGORY_EDIT, true)
                intent.putExtra(ExtraIntents.CATEGORY_EDIT_URL, url)
                intent.putExtra(ExtraIntents.CATEGORY_EDIT_IMAGE_URI, uri)
                intent.putExtra(ExtraIntents.CATEGORY_EDIT_NAME, categoryName)
                startActivityForResult(intent, ExtraIntents.CATEGORY_MESSAGE_CODE)
            } else {
                Snacker.show(mainLayout, getString(R.string.category_admin_page_category_unknown_edit_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.category_admin_page_edit_problem))
        }
    }


    private fun showDialog(url: String, imageName: String) {
        mCategoryUrl = url
        mImageName = imageName
        CategoryAdminDeleteDialog().show(fragmentManager, CATEGORY_DIALOG)
    }

    override fun onClickDelete() {
        if (viewModel != null && mCategoryUrl != null && mImageName != null) {
            if (NetworkUtil.isNetworkAvailable(this)) {
                launch(UI) {
                    val deleteImage = viewModel!!.deleteCategory(mCategoryUrl!!, mImageName!!)
                    Snacker.show(mainLayout, getString(deleteImage.messageResource))
                }
            } else {
                Snacker.show(mainLayout, getString(R.string.category_admin_page_delete_problem))
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

}