package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.category.category_add_edit

import android.app.Activity
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import com.bumptech.glide.Glide
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.explore.CategoryViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_category_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.android.synthetic.main.view_upload_button.*
import kotlinx.coroutines.experimental.launch


/**
 * Created by pavol.podstreleny on 07.01.2018.
 */
class CategoryEditActivity : AppCompatActivity(),INetworkChecker {

    private var isImageChanged = false

    private val IS_IMAGE_CHANGED = "image_changed"
    private val URI_UPLOADED_IMAGE = "uploaded_image"

    private val INTENT_TYPE = "image/png"

    private val RC_PHOTO_PICKER = 11

    private var mOldImageUri: String = ""
    private var mImageUri: Uri? = null
    private var mUrl: String = ""
    private var isAddRequest = true

    override var connectionObserver: ConnectionObserver? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_category_edit)

        setSupportActionBar(toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val connectionReceiver = ConnectionReceiver(noInternet)
        connectionObserver = ConnectionObserver(connectionReceiver,this,this)

        if (savedInstanceState != null && savedInstanceState.containsKey(URI_UPLOADED_IMAGE)) {
            if (savedInstanceState.containsKey(IS_IMAGE_CHANGED)) {
                isImageChanged = true
            }
            mImageUri = Uri.parse(savedInstanceState.getString(URI_UPLOADED_IMAGE))
            showImage(mImageUri!!)
        }

        val editTextData = EditableTextModel(
                hints = getString(R.string.category_admin_add_page_edittext_hint),
                textHint = getString(R.string.category_admin_add_page_text_restriction))

        val extras = intent.extras

        if (extras != null) {
            if (extras.containsKey(ExtraIntents.CATEGORY_ADD)) {
                supportActionBar?.title = resources.getString(R.string.category_admin_add_page_title)
            } else if (extras.containsKey(ExtraIntents.CATEGORY_EDIT)
                    && extras.containsKey(ExtraIntents.CATEGORY_EDIT_URL)
                    && extras.containsKey(ExtraIntents.CATEGORY_EDIT_IMAGE_URI)
                    && extras.containsKey(ExtraIntents.CATEGORY_EDIT_NAME)) {
                supportActionBar?.title = resources.getString(R.string.category_admin_page_category_edit_title)
                mOldImageUri = extras.getString(ExtraIntents.CATEGORY_EDIT_IMAGE_URI)
                if (!isImageChanged) {
                    showImage(Uri.parse(mOldImageUri))
                }

                editTextData.editTextValue = extras.getString(ExtraIntents.CATEGORY_EDIT_NAME)

                isAddRequest = false
                buttonAdd.text = resources.getString(R.string.global_button_edit)
                mUrl = extras.getString(ExtraIntents.CATEGORY_EDIT_URL)

            }
        }

        textInput.setUpModel(editTextData)

        val viewModel = ViewModelProviders.of(this).get(CategoryViewModel::class.java)

        buttonUpload.setOnClickListener {
            startImagePicker()
        }

        buttonAdd.setOnClickListener {
            if (connectionReceiver.isNetworkAvailable(this)) {
                checkCategory(viewModel)
            }else{
                Snacker.show(mainLayout,getString(R.string.category_admin_page_add_problem))
            }
        }

    }



    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == RC_PHOTO_PICKER && resultCode == RESULT_OK) {
            mImageUri = data?.data
            if (mImageUri != null) {
                isImageChanged = true
                showImage(mImageUri!!)
            }
        }
    }

    private fun startImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = INTENT_TYPE
        intent.putExtra(Intent.EXTRA_LOCAL_ONLY, true)
        startActivityForResult(Intent.createChooser(intent, getString(R.string.category_admin_add_page_picker_title)), RC_PHOTO_PICKER)
    }

    private fun checkCategory(viewModel: CategoryViewModel) {
        var categoryPassed = false
        var imagePassed = false

        if (textInput.isModelValid()) {
            categoryPassed = true
        } else {
            textInput.textInputEditText.error = getString(R.string.category_admin_add_page_text_restriction)
            return
        }

        if (mImageUri != null || mOldImageUri != "") {
            imagePassed = true
        }

        if(mImageUri == null){
            Snacker.show(mainLayout,getString(R.string.category_admin_add_category_image_problem))
        }

        if (imagePassed && categoryPassed) {
            var message = ""
            progressBar.visibility = View.VISIBLE

                launch {
                    if (isAddRequest) {
                        val addCategory = viewModel.addCategory(mImageUri!!, textInput.getTextInputData())
                        message = getString(addCategory.messageResource)
                    } else {
                        val editCategory = viewModel.editCategory(isImageChanged, mImageUri, mOldImageUri, textInput.getTextInputData(), mUrl)
                        message = getString(editCategory.messageResource)
                    }
                    val intent = Intent()
                    intent.putExtra(ExtraIntents.CATEGORY_MESSAGE, message)
                    setResult(Activity.RESULT_OK, intent)
                    finish()
                }

        }
    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun showImage(uri: Uri) {
        uploadedImageView.visibility = View.VISIBLE
        Glide.with(this).load(uri).into(uploadedImageView)
    }

    override fun onStop() {
        super.onStop()
        progressBar.visibility = View.GONE
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mImageUri != null) {
            outState?.putString(URI_UPLOADED_IMAGE, mImageUri.toString())
        }

        if (isImageChanged) {
            outState?.putBoolean(IS_IMAGE_CHANGED, isImageChanged)
        }

    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }
}