package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.Action
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Exercise
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.exercise_add.ExerciseAddActivity
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.exercise_edit.ExerciseEditActivity
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_exercise.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_data.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 15.01.2018.
 */
class ExerciseAdminActivity : AppCompatActivity(),
        ExerciseAdminAdapter.OnClickAdminGroupListener,
        ExerciseAdminDeleteDialog.OnDialogClick,
        INetworkChecker {

    private val EXERCISE_DIALOG = "exercise_dialog"
    private val UNKNOWN = "unknown"

    //Constants for screen rotations
    private val CATEGORY_SELECTED_ID = "CATEGORY_SELECTED_ID"
    private val CATEGORY_SELECTED_VALUE = "CATEGORY_SELECTED_VALUE"
    private val TOPIC_SELECTED_ID = "TOPIC_SELECTED_ID"
    private val TOPIC_SELECTED_VALUE = "TOPIC_SELECTED_VALUE"
    private val GROUP_SELECTED_ID = "GROUP_SELECTED_ID"
    private val GROUP_SELECTED_VALUE = "GROUP_SELECTED_VALUE"


    private var mCategorySelectedID: Int? = null
    private var mCategorySelectedValue: String? = null
    private var mTopicSelectedID: Int? = null
    private var mTopicSelectedValue: String? = null
    private var mGroupSelectedID: Int? = null
    private var mGroupSelectedValue: String? = null

    var mExerciseID: String? = null
    var viewModel: ExerciseAdminViewModel? = null


    private var isRotatedCategory = false
    private var isRotatedTopic = false
    private var isRotatedGroup = false

    override var connectionObserver: ConnectionObserver? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_exercise)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.exercise_admin_page_group_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)



        if (savedInstanceState != null && savedInstanceState.containsKey(CATEGORY_SELECTED_ID) && savedInstanceState.containsKey(CATEGORY_SELECTED_VALUE)) {
            mCategorySelectedValue = savedInstanceState.getString(CATEGORY_SELECTED_VALUE)
            mCategorySelectedID = savedInstanceState.getInt(CATEGORY_SELECTED_ID)
            isRotatedCategory = true
        }
        if (savedInstanceState != null && savedInstanceState.containsKey(TOPIC_SELECTED_ID) && savedInstanceState.containsKey(TOPIC_SELECTED_VALUE)) {
            mTopicSelectedValue = savedInstanceState.getString(TOPIC_SELECTED_VALUE)
            mTopicSelectedID = savedInstanceState.getInt(TOPIC_SELECTED_ID)
            isRotatedTopic = true
        }

        if (savedInstanceState != null && savedInstanceState.containsKey(GROUP_SELECTED_VALUE) && savedInstanceState.containsKey(GROUP_SELECTED_ID)) {
            mGroupSelectedID = savedInstanceState.getInt(GROUP_SELECTED_ID)
            mGroupSelectedValue = savedInstanceState.getString(GROUP_SELECTED_VALUE)
            isRotatedGroup = true
        }


        val categoryAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerCategoryData = EditableTextModel(getString(R.string.spinner_category_title))
        categorySpinner.setUpModel(spinnerCategoryData)

        val topicAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerTopicData = EditableTextModel(getString(R.string.spinner_topic_title))
        topicSpinner.setUpModel(spinnerTopicData)

        val groupAdatper = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerGroupData = EditableTextModel(getString(R.string.spinner_group_title))
        groupSpinner.setUpModel(spinnerGroupData)

        val exerciseAdapter = ExerciseAdminAdapter(this)
        exerciseAdminRecyclerView.layoutManager = LinearLayoutManager(this)
        exerciseAdminRecyclerView.adapter = exerciseAdapter

        categorySpinner.spinner.adapter = categoryAdapter


        viewModel = ViewModelProviders.of(this).get(ExerciseAdminViewModel::class.java)

        viewModel?.getAllCategories()?.observe(this, Observer { data ->
            categoryAdapter.swapData(data as ArrayList<Selectable>)
            if (mCategorySelectedID != null) {
                categorySpinner.spinner.setSelection(mCategorySelectedID!!)
            }
            loaderProgress.visibility = View.VISIBLE
        })


        viewModel?.topics?.observe(this, Observer { data ->
            topicAdapter.swapData(data as ArrayList<Selectable>)
            topicSpinner.spinner.adapter = topicAdapter
            if (data.isEmpty()) {
                loaderProgress.visibility = View.GONE
                noAvailableData.visibility = View.VISIBLE
                mTopicSelectedValue = null
                mTopicSelectedID = null
            } else {
                noAvailableData.visibility = View.GONE
            }
            if (mTopicSelectedID != null) {
                topicSpinner.spinner.setSelection(mTopicSelectedID!!)
            }
            groupAdatper.clearData()
        })

        viewModel?.groups?.observe(this, Observer { data ->
            groupAdatper.swapData(data as ArrayList<Selectable>)
            groupSpinner.spinner.adapter = groupAdatper
            if (data.isEmpty()) {
                loaderProgress.visibility = View.GONE
                noAvailableData.visibility = View.VISIBLE
                mGroupSelectedValue = null
                mGroupSelectedID = null
            } else {
                noAvailableData.visibility = View.GONE
            }
            if (mGroupSelectedID != null) {
                groupSpinner.spinner.setSelection(mGroupSelectedID!!)
            }

            exerciseAdapter.clearData()
        })

        viewModel?.exercises?.observe(this, Observer { data ->
            loaderProgress.visibility = View.GONE
            if (data != null) {
                exerciseAdapter.swapData(data)
                if (data.size == 0) {
                    noAvailableData.visibility = View.VISIBLE
                    exerciseAdminRecyclerView.visibility = View.GONE
                } else {
                    noAvailableData.visibility = View.GONE
                    exerciseAdminRecyclerView.visibility = View.VISIBLE
                }
            }

        })


        setUpSpinnerGroupListener(groupAdatper)
        setUpSpinnerTopicListener(topicAdapter)
        setUpCategorySpinnerListener(categoryAdapter)


        addGroup.setOnClickListener {
            startAddGroupActivity()
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun startAddGroupActivity() {
        if (mGroupSelectedValue != null && mTopicSelectedValue != null && mCategorySelectedValue != null) {
            if (NetworkUtil.isNetworkAvailable(this)) {
                val intent = Intent(this, ExerciseAddActivity::class.java)
                intent.putExtra(ExtraIntents.EXERCISE_ADD, true)
                intent.putExtra(ExtraIntents.EXERCISE_GROUP_ID, mGroupSelectedValue)
                startActivityForResult(intent, ExtraIntents.EXERCISE_MESSAGE_CODE)
            } else {
                Snacker.show(mainLayout, getString(R.string.exercise_admin_page_add_internet_problem))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.exercise_admin_page_add_problem))
        }
    }

    private fun setUpCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter) {
        categorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategorySelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedCategory) {
                    mCategorySelectedID = position
                    mCategorySelectedValue = categoryAdapter.getActualitem(position).getPathID()
                    mTopicSelectedID = null
                    mTopicSelectedValue = null
                }
                viewModel?.categoryID?.value = mCategorySelectedValue
                if (isRotatedCategory) {
                    isRotatedCategory = false
                }
            }
        }
    }

    private fun setUpSpinnerTopicListener(topicAdapter: DefaultSpinnerAdapter) {
        topicSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                mTopicSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedTopic) {
                    mTopicSelectedID = position
                    mTopicSelectedValue = topicAdapter.getActualitem(position).getPathID()
                    mGroupSelectedID = null
                    mGroupSelectedValue = null
                }
                viewModel?.topicID?.value = mTopicSelectedValue
                if (isRotatedTopic) {
                    isRotatedTopic = false
                }
            }
        }
    }

    private fun setUpSpinnerGroupListener(groupAdatper: DefaultSpinnerAdapter) {
        groupSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mGroupSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedGroup) {
                    mGroupSelectedID = position
                    mGroupSelectedValue = groupAdatper.getActualitem(position).getPathID()
                }
                viewModel?.groupID?.value = mGroupSelectedValue
                if (isRotatedGroup) {
                    isRotatedGroup = false
                }
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ExtraIntents.EXERCISE_MESSAGE_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                Snacker.show(mainLayout, data?.getStringExtra(ExtraIntents.EXERCISES_MESSAGE).toString())
            }
        }
    }


    override fun onClickDelete() {
        if (mExerciseID != null) {
            launch(UI) {
                val deletedGroup = viewModel?.deleteExercise(mExerciseID!!)
                Snacker.show(mainLayout, getString(deletedGroup!!.messageResource))
            }
        }
    }

    override fun onClick(exercise: Exercise, action: Action) {
        when (action) {
            Action.DELETE -> deleteGroup(exercise)
            Action.EDIT -> editExercise(exercise)
        }
    }

    private fun deleteGroup(exercise: Exercise) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (exercise.exerciseKey != UNKNOWN) {
                mExerciseID = exercise.getPathID()
                ExerciseAdminDeleteDialog().show(fragmentManager, EXERCISE_DIALOG)
            } else {
                Snacker.show(mainLayout, getString(R.string.exercise_admin_page_unknown_delete_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.exercise_admin_page_delete_internet_problem))
        }
    }

    private fun editExercise(exercise: Exercise) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (exercise.exerciseKey != UNKNOWN) {
                val intent = Intent(this, ExerciseEditActivity::class.java)
                intent.putExtra(ExtraIntents.EXERCISE_EDIT, true)
                intent.putExtra(ExtraIntents.EXERCISE_TOPIC_ID, mTopicSelectedValue)
                intent.putExtra(ExtraIntents.EXERCISE_CATEGORY_ID, mCategorySelectedValue)
                intent.putExtra(ExtraIntents.EXERCISE_GROUP_ID, mGroupSelectedValue)
                intent.putExtra(ExtraIntents.EXERCISE_TOPIC_SELECTED, mTopicSelectedID)
                intent.putExtra(ExtraIntents.EXERCISE_CATEGORY_SELECTED, mCategorySelectedID)
                intent.putExtra(ExtraIntents.EXERCISE_GROUP_SELECTED, mGroupSelectedID)
                intent.putExtra(ExtraIntents.EXERCISE_ID, exercise.exerciseKey)
                intent.putExtra(ExtraIntents.EXERCISE_DESCRIPTION, exercise.description)
                startActivityForResult(intent, ExtraIntents.EXERCISE_MESSAGE_CODE)
            } else {
                Snacker.show(mainLayout, getString(R.string.exercise_admin_page_unknown_edit_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.exercise_admin_page_edit_internet_problem))
        }
    }


    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mCategorySelectedID != null && mCategorySelectedValue != null) {
            outState?.putInt(CATEGORY_SELECTED_ID, mCategorySelectedID!!)
            outState?.putString(CATEGORY_SELECTED_VALUE, mCategorySelectedValue)
        }

        if (mTopicSelectedValue != null && mTopicSelectedID != null) {
            outState?.putString(TOPIC_SELECTED_VALUE, mTopicSelectedValue)
            outState?.putInt(TOPIC_SELECTED_ID, mTopicSelectedID!!)
        }

        if (mGroupSelectedValue != null && mGroupSelectedID != null) {
            outState?.putString(GROUP_SELECTED_VALUE, mGroupSelectedValue)
            outState?.putInt(GROUP_SELECTED_ID, mGroupSelectedID!!)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }
}