package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.Transformations
import android.arch.lifecycle.ViewModel
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult
import cz.uhk.pavolpodstreleny.schoolproject.data.repositories.ExerciseAdminRepository
import cz.uhk.pavolpodstreleny.schoolproject.model.*
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.SingleLiveEvent

/**
 * Created by pavol.podstreleny on 15.01.2018.
 */
class ExerciseAdminViewModel : ViewModel() {


    private var categories: SingleLiveEvent<List<Category>>? = null


    private val mExerciseAdminRepository = ExerciseAdminRepository()


    var categoryID: MutableLiveData<String> = MutableLiveData()
    val topics: LiveData<List<Topic>> = Transformations.switchMap(categoryID, { id -> getAllTopicsInCategory(id) }
    )

    var topicID: MutableLiveData<String> = MutableLiveData()
    val groups: LiveData<List<Group>> = Transformations.switchMap(topicID, { id ->
        getAllGroupsInTopic(id)
    })

    var groupID: MutableLiveData<String> = MutableLiveData()
    val exercises: LiveData<ArrayList<Exercise>> = Transformations.switchMap(groupID, { id -> getAllExercisesInTopic(id) }
    )

    var userID: MutableLiveData<String> = MutableLiveData()
    val user: LiveData<UserProfile> = Transformations.switchMap(userID, {
        id -> getCurrentUser(id)
    })

    fun getAllCategories(): SingleLiveEvent<List<Category>> {
        if (categories == null) {
            categories = mExerciseAdminRepository.getAllCategoriesForSpinner()
        }
        return categories!!
    }

    fun getCurrentUser(childUID: String): LiveData<UserProfile> {
        return mExerciseAdminRepository.getUser(childUID)
    }

    fun getAllTopicsInCategory(id: String): SingleLiveEvent<List<Topic>> {
        return mExerciseAdminRepository.getAllTopicsInCategorySpinner(id)
    }

    fun getAllGroupsInTopic(id: String): SingleLiveEvent<List<Group>> {
        return mExerciseAdminRepository.getAllGroupsInTopicSpinner(id)
    }

    fun getAllExercisesInTopic(id: String): LiveData<ArrayList<Exercise>> {
        return mExerciseAdminRepository.getAllExercisesInTopic(id)
    }

    suspend fun deleteExercise(groupID: String): FirebaseResult {
        return mExerciseAdminRepository.deleteExercise(groupID)
    }

    suspend fun addExercise(description: String, groupID: String?): FirebaseResult {
        return mExerciseAdminRepository.addExercise(description, groupID)
    }

    suspend fun editExercise(groupID: String?, exerciseID: String?, description: String): FirebaseResult {
        return mExerciseAdminRepository.editExercise(groupID, exerciseID, description)
    }


}