package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.exercise_add

import android.app.Activity
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.ExerciseAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_category_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.android.synthetic.main.view_upload_button.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 15.01.2018.
 */
class ExerciseAddActivity : AppCompatActivity(), INetworkChecker {

    private var mGroupID: String? = null
    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_category_edit)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        setSupportActionBar(toolbar)
        supportActionBar?.title = getString(R.string.exercise_admin_add_page_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val editTextData = EditableTextModel(
                hints = getString(R.string.exercise_admin_add_page_edittext_hint),
                textHint = getString(R.string.exercise_admin_add_page_text_restriction))

        val extras = intent.extras
        if (extras != null) {
            if (extras.containsKey(ExtraIntents.EXERCISE_ADD) && extras.containsKey(ExtraIntents.EXERCISE_GROUP_ID)) {
                mGroupID = extras.getString(ExtraIntents.EXERCISE_GROUP_ID)
            }
        }

        textInput.setUpModel(editTextData)

        uploadedImageView.visibility = View.GONE
        buttonUpload.visibility = View.GONE

        val viewModel = ViewModelProviders.of(this).get(ExerciseAdminViewModel::class.java)

        buttonAdd.setOnClickListener {
            if (NetworkUtil.isNetworkAvailable(this)) {
                createExercise(viewModel)
            } else {
                Snacker.show(mainLayout, getString(R.string.exercise_admin_page_add_internet_problem))
            }
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun createExercise(viewModel: ExerciseAdminViewModel) {
        var topicPassed = false

        if (textInput.isModelValid()) {
            topicPassed = true
        } else {
            textInput.textInputEditText.error = getString(R.string.exercise_admin_add_page_text_restriction)
            return
        }

        if (topicPassed) {

            launch(UI) {
                progressBar.visibility = View.VISIBLE
                val addGroup = viewModel.addExercise(textInput.getTextInputData(), mGroupID)
                val message = getString(addGroup.messageResource)
                val intent = Intent()
                intent.putExtra(ExtraIntents.EXERCISES_MESSAGE, message)
                progressBar.visibility = View.GONE
                setResult(Activity.RESULT_OK, intent)
                finish()
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

}