package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.exercise_edit

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.ExerciseAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil.isNetworkAvailable
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import cz.uhk.pavolpodstreleny.schoolproject.utils.UtilChecker
import kotlinx.android.synthetic.main.activity_admin_exercise_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch
import java.util.*

/**
 * Created by pavol.podstreleny on 16.01.2018.
 */
class ExerciseEditActivity : AppCompatActivity(), INetworkChecker {

    private val CATEGORY_SELECTED_ID = "CATEGORY_SELECTED_ID"
    private val CATEGORY_SELECTED_VALUE = "CATEGORY_SELECTED_VALUE"
    private val TOPIC_SELECTED_ID = "TOPIC_SELECTED_ID"
    private val TOPIC_SELECTED_VALUE = "TOPIC_SELECTED_VALUE"
    private val GROUP_SELECTED_ID = "GROUP_SELECTED_ID"
    private val GROUP_SELECTED_VALUE = "GROUP_SELECTED_VALUE"


    private var mCategorySelectedID: Int? = null
    private var mCategorySelectedValue: String? = null
    private var mTopicSelectedID: Int? = null
    private var mTopicSelectedValue: String? = null
    private var mGroupSelectedID: Int? = null
    private var mGroupSelectedValue: String? = null

    var mExerciseID: String? = null
    var viewModel: ExerciseAdminViewModel? = null

    private var isRotatedCategory = false
    private var isRotatedTopic = false
    private var isRotatedGroup = false

    private var firstRun = true

    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_exercise_edit)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.exercise_admin_edit_page_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        buttonAdd.text = getString(R.string.global_button_edit)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)


        if (savedInstanceState != null) {
            firstRun = false
        }
        if (savedInstanceState != null && savedInstanceState.containsKey(CATEGORY_SELECTED_ID) && savedInstanceState.containsKey(CATEGORY_SELECTED_VALUE)) {
            mCategorySelectedValue = savedInstanceState.getString(CATEGORY_SELECTED_VALUE)
            mCategorySelectedID = savedInstanceState.getInt(CATEGORY_SELECTED_ID)
            isRotatedCategory = true
        }
        if (savedInstanceState != null && savedInstanceState.containsKey(TOPIC_SELECTED_ID) && savedInstanceState.containsKey(TOPIC_SELECTED_VALUE)) {
            mTopicSelectedValue = savedInstanceState.getString(TOPIC_SELECTED_VALUE)
            mTopicSelectedID = savedInstanceState.getInt(TOPIC_SELECTED_ID)
            isRotatedTopic = true
        }

        if (savedInstanceState != null && savedInstanceState.containsKey(GROUP_SELECTED_VALUE) && savedInstanceState.containsKey(GROUP_SELECTED_ID)) {
            mGroupSelectedID = savedInstanceState.getInt(GROUP_SELECTED_ID)
            mGroupSelectedValue = savedInstanceState.getString(GROUP_SELECTED_VALUE)
            isRotatedGroup = true
        }

        val categoryAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerCategoryData = EditableTextModel(getString(R.string.spinner_category_title))
        categorySpinner.setUpModel(spinnerCategoryData)

        val topicAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerTopicData = EditableTextModel(getString(R.string.spinner_topic_title))
        topicSpinner.setUpModel(spinnerTopicData)

        val groupAdatper = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerGroupData = EditableTextModel(getString(R.string.spinner_group_title))
        groupSpinner.setUpModel(spinnerGroupData)


        categorySpinner.spinner.adapter = categoryAdapter


        val viewModel = ViewModelProviders.of(this).get(ExerciseAdminViewModel::class.java)

        viewModel.getAllCategories().observe(this, Observer { data ->
            categoryAdapter.swapData(data as ArrayList<Selectable>)
            if (mCategorySelectedID != null) {
                categorySpinner.spinner.setSelection(mCategorySelectedID!!)
            }
        })

        viewModel.topics.observe(this, Observer { data ->
            topicSpinner.spinner.adapter = topicAdapter
            topicAdapter.swapData(data as ArrayList<Selectable>)
            if (data.isEmpty()) {
                mTopicSelectedValue = null
                mTopicSelectedID = null
            }
            if (mTopicSelectedID != null) {
                topicSpinner.spinner.setSelection(mTopicSelectedID!!)
            }
            groupAdatper.clearData()
        })

        viewModel.groups.observe(this, Observer { data ->
            groupSpinner.spinner.adapter = groupAdatper
            groupAdatper.swapData(data as ArrayList<Selectable>)
            if (data.isEmpty()) {
                mGroupSelectedValue = null
                mGroupSelectedID = null
            }
            if (mGroupSelectedID != null) {
                groupSpinner.spinner.setSelection(mGroupSelectedID!!)
            }
        })


        setCategorySpinnerListener(categoryAdapter, viewModel)
        setTopicSpinnerListener(topicAdapter, viewModel)
        setGroupSpinnerListener(groupAdatper)
        if (firstRun) {
            setProperDataFromLaunchActivity()
            firstRun = false
        }
        buttonAdd.setOnClickListener {
            if (mGroupSelectedValue != null && mTopicSelectedValue != null && mCategorySelectedValue != null) {
                if (isNetworkAvailable(this)) {
                    editExercise(viewModel)
                } else {
                    Snacker.show(mainLayout, getString(R.string.exercise_admin_page_edit_internet_problem))
                }
            } else {
                Snacker.show(mainLayout, getString(R.string.exercise_admin_page_edit_problem))
            }
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun editExercise(viewModel: ExerciseAdminViewModel) {

        if (UtilChecker.hasCorrectData(textInput.isModelValid())) {
            launch(UI) {
                progressBar.visibility = View.VISIBLE
                val editExercise = viewModel.editExercise(mGroupSelectedValue, mExerciseID, textInput.getTextInputData())
                progressBar.visibility = View.INVISIBLE
                val intent = Intent()
                intent.putExtra(ExtraIntents.EXERCISES_MESSAGE, getString(editExercise.messageResource))
                setResult(Activity.RESULT_OK, intent)
                finish()
            }
        } else {
            textInput.textInputEditText.error = textInput.hintTextView.text
        }
    }

    private fun setGroupSpinnerListener(groupAdapter: DefaultSpinnerAdapter) {
        groupSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                mGroupSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedGroup) {
                    mGroupSelectedID = position
                    mGroupSelectedValue = groupAdapter.getActualitem(position).getPathID()
                }
                if (isRotatedGroup) {
                    isRotatedGroup = false
                }
            }
        }
    }


    private fun setCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter, viewModel: ExerciseAdminViewModel) {
        categorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategorySelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedCategory) {
                    mCategorySelectedID = position
                    mCategorySelectedValue = categoryAdapter.getActualitem(position).getPathID()
                }
                viewModel.categoryID.value = mCategorySelectedValue
                if (isRotatedCategory) {
                    isRotatedCategory = false
                }
            }
        }
    }

    private fun setTopicSpinnerListener(topicAdapter: DefaultSpinnerAdapter, viewModel: ExerciseAdminViewModel) {
        topicSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mTopicSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedTopic) {
                    mTopicSelectedID = position
                    mTopicSelectedValue = topicAdapter.getActualitem(position).getPathID()
                }
                viewModel.topicID.value = mTopicSelectedValue
                if (isRotatedTopic) {
                    isRotatedTopic = false
                }
            }
        }
    }

    private fun setProperDataFromLaunchActivity() {
        val extra = intent.extras
        if (extra != null) {
            if (extra.containsKey(ExtraIntents.EXERCISE_EDIT) &&
                    extra.containsKey(ExtraIntents.EXERCISE_TOPIC_ID) &&
                    extra.containsKey(ExtraIntents.EXERCISE_CATEGORY_ID) &&
                    extra.containsKey(ExtraIntents.EXERCISE_GROUP_ID) &&
                    extra.containsKey(ExtraIntents.EXERCISE_TOPIC_SELECTED) &&
                    extra.containsKey(ExtraIntents.EXERCISE_CATEGORY_SELECTED) &&
                    extra.containsKey(ExtraIntents.EXERCISE_GROUP_SELECTED) &&
                    extra.containsKey(ExtraIntents.EXERCISE_ID) &&
                    extra.containsKey(ExtraIntents.EXERCISE_DESCRIPTION)) {

                mTopicSelectedValue = extra.getString(ExtraIntents.EXERCISE_TOPIC_ID)
                mCategorySelectedValue = extra.getString(ExtraIntents.EXERCISE_CATEGORY_ID)
                mGroupSelectedValue = extra.getString(ExtraIntents.EXERCISE_GROUP_ID)

                mTopicSelectedID = extra.getInt(ExtraIntents.EXERCISE_TOPIC_SELECTED)
                mCategorySelectedID = extra.getInt(ExtraIntents.EXERCISE_CATEGORY_SELECTED)
                mGroupSelectedID = extra.getInt(ExtraIntents.EXERCISE_GROUP_SELECTED)
                mExerciseID = extra.getString(ExtraIntents.EXERCISE_ID)

                val textModelData = EditableTextModel(getString(R.string.exercise_admin_add_page_edittext_hint),
                        extra.getString(ExtraIntents.EXERCISE_DESCRIPTION),
                        getString(R.string.exercise_admin_add_page_text_restriction))
                textInput.setUpModel(textModelData)

            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mCategorySelectedID != null && mCategorySelectedValue != null) {
            outState?.putInt(CATEGORY_SELECTED_ID, mCategorySelectedID!!)
            outState?.putString(CATEGORY_SELECTED_VALUE, mCategorySelectedValue)
        }

        if (mTopicSelectedValue != null && mTopicSelectedID != null) {
            outState?.putString(TOPIC_SELECTED_VALUE, mTopicSelectedValue)
            outState?.putInt(TOPIC_SELECTED_ID, mTopicSelectedID!!)
        }

        if (mGroupSelectedValue != null && mGroupSelectedID != null) {
            outState?.putString(GROUP_SELECTED_VALUE, mGroupSelectedValue)
            outState?.putInt(GROUP_SELECTED_ID, mGroupSelectedID!!)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }


}