package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.Action
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Group
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups.group_add.GroupAddActivity
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups.group_edit.GroupEditActivity
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_group.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_data.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 13.01.2018.
 */

class GroupAdminActivity : AppCompatActivity(), GroupAdminAdapter.OnClickAdminGroupListener,
        GroupAdminDeleteDialog.OnDialogClick,
        INetworkChecker {

    val GROUP_DIALOG = "group_dialog"

    var mGroupdID: String? = null
    val UNKNOWN = "unknown"

    val CATEGORY_SELECTED_ID = "category_selected_id"
    val CATEGORY_SELECTED_VALUE = "category_selected_value"
    val TOPIC_SELECTED_ID = "topic_selected_id"
    val TOPIC_SELECTED_VALUE = "category_selected_value"

    var mCategorySelectedID: Int? = null
    var mCategorySelectedValue: String? = null
    var mTopicSelectedID: Int? = null
    var mTopicSelectedValue: String? = null

    var isRotated = false
    var viewModel: GroupAdminViewModel? = null
    override var connectionObserver: ConnectionObserver? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_group)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.group_admin_page_group_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        if (savedInstanceState != null && savedInstanceState.containsKey(CATEGORY_SELECTED_ID) && savedInstanceState.containsKey(CATEGORY_SELECTED_VALUE)) {
            mCategorySelectedValue = savedInstanceState.getString(CATEGORY_SELECTED_VALUE)
            mCategorySelectedID = savedInstanceState.getInt(CATEGORY_SELECTED_ID)
            isRotated = true
        }
        if (savedInstanceState != null && savedInstanceState.containsKey(TOPIC_SELECTED_ID) && savedInstanceState.containsKey(TOPIC_SELECTED_VALUE)) {
            mTopicSelectedValue = savedInstanceState.getString(TOPIC_SELECTED_VALUE)
            mTopicSelectedID = savedInstanceState.getInt(TOPIC_SELECTED_ID)
        }


        val categoryAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        categorySpinner.spinner.adapter = categoryAdapter
        val spinnerCategoryData = EditableTextModel(getString(R.string.spinner_category_title))
        categorySpinner.setUpModel(spinnerCategoryData)


        val topicAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerTopicData = EditableTextModel(getString(R.string.spinner_topic_title))
        topicSpinner.setUpModel(spinnerTopicData)


        val groupAdatper = GroupAdminAdapter(this)
        groupAdminRecyclerView.adapter = groupAdatper
        groupAdminRecyclerView.layoutManager = LinearLayoutManager(this)

        viewModel = ViewModelProviders.of(this).get(GroupAdminViewModel::class.java)

        viewModel?.getAllCategories()?.observe(this, Observer { data ->
            categoryAdapter.swapData(data as ArrayList<Selectable>)
            if (mCategorySelectedID != null) {
                categorySpinner.spinner.setSelection(mCategorySelectedID!!)
            }
            loaderProgress.visibility = View.VISIBLE
        })

        viewModel?.topics?.observe(this, Observer { data ->
            topicAdapter.swapData(data as ArrayList<Selectable>)
            topicSpinner.spinner.adapter = topicAdapter
            if (data.size == 0) {
                loaderProgress.visibility = View.GONE
                noAvailableData.visibility = View.VISIBLE
            } else {
                noAvailableData.visibility = View.GONE
            }
            if (mTopicSelectedID != null) {
                topicSpinner.spinner.setSelection(mTopicSelectedID!!)
            }
            groupAdatper.clearData()
        })

        viewModel?.groups?.observe(this, Observer { data ->
            loaderProgress.visibility = View.GONE
            if (data != null) {
                groupAdatper.swapData(data)
                if (data.size == 0) {
                    noAvailableData.visibility = View.VISIBLE
                    groupAdminRecyclerView.visibility = View.GONE
                } else {
                    noAvailableData.visibility = View.GONE
                    groupAdminRecyclerView.visibility = View.VISIBLE
                }
            }
        })


        setUpCategorySpinnerListener(categoryAdapter, viewModel!!)
        setUpTopicSpinnerListener(topicAdapter, viewModel!!)


        addGroup.setOnClickListener {
            if (NetworkUtil.isNetworkAvailable(this)) {
                startAddGroupActivity()
            } else {
                Snacker.show(mainLayout, getString(R.string.group_admin_page_add_internet_problem))
            }
        }


    }

    override fun onStart() {
        super.onStart()
        connectionObserver?.start()
    }

    private fun startAddGroupActivity() {
        if (mTopicSelectedValue == null) {
            Snacker.show(mainLayout, getString(R.string.group_admin_page_group_add_problem))
        } else {
            val intent = Intent(this, GroupAddActivity::class.java)
            intent.putExtra(ExtraIntents.GROUP_ADD, true)
            intent.putExtra(ExtraIntents.GROUP_TOPIC_ID, mTopicSelectedValue)
            startActivityForResult(intent, ExtraIntents.GROUP_MESSAGE_CODE)
        }


    }


    private fun setUpCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter, viewModel: GroupAdminViewModel) {
        categorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategorySelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                mCategorySelectedID = position
                mCategorySelectedValue = categoryAdapter.getActualitem(position).getPathID()
                viewModel.categoryID.value = mCategorySelectedValue
                if (!isRotated) {
                    mTopicSelectedValue = null
                    mTopicSelectedID = null
                } else {
                    isRotated = false
                }
            }
        }
    }

    private fun setUpTopicSpinnerListener(topicAdapter: DefaultSpinnerAdapter, viewModel: GroupAdminViewModel) {
        topicSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                mTopicSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                mTopicSelectedID = position
                mTopicSelectedValue = topicAdapter.getActualitem(position).getPathID()
                viewModel.topicID.value = mTopicSelectedValue
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ExtraIntents.GROUP_MESSAGE_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                Snacker.show(mainLayout, data?.getStringExtra(ExtraIntents.GROUP_MESSAGE).toString())
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mCategorySelectedID != null && mCategorySelectedValue != null) {
            outState?.putInt(CATEGORY_SELECTED_ID, mCategorySelectedID!!)
            outState?.putString(CATEGORY_SELECTED_VALUE, mCategorySelectedValue)
        }

        if (mTopicSelectedValue != null && mTopicSelectedID != null) {
            outState?.putString(TOPIC_SELECTED_VALUE, mTopicSelectedValue)
            outState?.putInt(TOPIC_SELECTED_ID, mTopicSelectedID!!)
        }
    }

    override fun onClickDelete() {
        if (mGroupdID != null) {
            launch(UI) {
                loaderProgress.visibility = View.VISIBLE
                val deletedGroup = viewModel?.deleteGroup(mGroupdID!!)
                Snacker.show(mainLayout, getString(deletedGroup!!.messageResource))
            }

        }
    }

    override fun onClick(group: Group, action: Action) {
        when (action) {
            Action.DELETE -> deleteGroup(group)
            Action.EDIT -> editGroup(group)
        }
    }

    private fun deleteGroup(group: Group) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (group.groupKey != UNKNOWN) {
                mGroupdID = group.getPathID()
                GroupAdminDeleteDialog().show(supportFragmentManager, GROUP_DIALOG)
            } else {
                Snacker.show(mainLayout, getString(R.string.group_admin_page_unknown_delete_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.group_admin_page_delete_internet_problem))
        }
    }

    private fun editGroup(group: Group) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (group.groupKey != UNKNOWN) {
                val intent = Intent(this, GroupEditActivity::class.java)
                intent.putExtra(ExtraIntents.GROUP_EDIT, true)
                intent.putExtra(ExtraIntents.GROUP_TOPIC_ID, mTopicSelectedValue)
                intent.putExtra(ExtraIntents.GROUP_CATEGORY_ID, mCategorySelectedValue)
                intent.putExtra(ExtraIntents.GROUP_TOPIC_SELECTED, mTopicSelectedID)
                intent.putExtra(ExtraIntents.GROUP_CATEGORY_SELECTED, mCategorySelectedID)
                intent.putExtra(ExtraIntents.GROUP_ID, group.groupKey)
                intent.putExtra(ExtraIntents.GROUP_DESCRIPTION, group.description)
                startActivityForResult(intent, ExtraIntents.GROUP_MESSAGE_CODE)
            } else {
                Snacker.show(mainLayout, getString(R.string.group_admin_page_unknown_edit_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.group_admin_page_edit_internet_problem))
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

}