package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.Transformations
import android.arch.lifecycle.ViewModel
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult
import cz.uhk.pavolpodstreleny.schoolproject.data.repositories.GroupAdminRepository
import cz.uhk.pavolpodstreleny.schoolproject.model.Category
import cz.uhk.pavolpodstreleny.schoolproject.model.Group
import cz.uhk.pavolpodstreleny.schoolproject.model.Topic
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.SingleLiveEvent

/**
 * Created by pavol.podstreleny on 13.01.2018.
 */

class GroupAdminViewModel : ViewModel() {


    private var categories: SingleLiveEvent<List<Category>>? = null


    private val mGroupAdminRepository = GroupAdminRepository()

    var categoryID: MutableLiveData<String> = MutableLiveData()
    val topics: LiveData<ArrayList<Topic>> = Transformations.switchMap(categoryID, { id -> getAllTopicsInCategory(id) })

    var topicID: MutableLiveData<String> = MutableLiveData()
    val groups: LiveData<ArrayList<Group>> = Transformations.switchMap(topicID, { id -> getAllGroupsInTopic(id) })

    fun getAllCategories(): SingleLiveEvent<List<Category>> {
        if (categories == null) {
            categories = mGroupAdminRepository.getAllCategoriesForSpinner()
        }
        return categories!!
    }

    fun getAllTopicsInCategory(id: String): LiveData<ArrayList<Topic>> {
        return mGroupAdminRepository.getAllTopicsInCategory(id)
    }

    fun getAllGroupsInTopic(id: String): LiveData<ArrayList<Group>> {
        return mGroupAdminRepository.getAllGroupsInTopic(id)
    }

    suspend fun deleteGroup(groupID: String): FirebaseResult {
        return mGroupAdminRepository.deleteGroup(groupID)
    }

    suspend fun addGroup(description: String, groupID: String?): FirebaseResult {
        return mGroupAdminRepository.addGroup(description, groupID)
    }

    suspend fun editGroup(topicID: String?, groupID: String?, description: String): FirebaseResult {
        return mGroupAdminRepository.editGroup(topicID, groupID, description)
    }


}