package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups.group_edit

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.util.Log
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups.GroupAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import cz.uhk.pavolpodstreleny.schoolproject.utils.UtilChecker
import kotlinx.android.synthetic.main.activity_admin_group_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch
import java.util.*

/**
 * Created by pavol.podstreleny on 15.01.2018.
 */
class GroupEditActivity : AppCompatActivity(), INetworkChecker {

    val CATEGORY_ID = "CATEGORY_ID"
    val CATEGORY_SELECTED_ID = "CATEGORY_SELECTED_ID"

    val TOPIC_ID = "TOPIC_ID"
    val TOPIC_SELECTED_ID = "TOPIC_SELECTED_ID"

    private var mCategoryID: String? = null
    private var mTopicID: String? = null
    private var mGroupID: String? = null

    private var mCategorySelectedID: Int? = null
    private var mTopicSelectedID: Int? = null


    private var isRotated = false
    private var isFirstTime = true

    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_group_edit)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.group_admin_edit_page_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        buttonAdd.text = getString(R.string.global_button_edit)


        val textModelData = EditableTextModel(hints = getString(R.string.group_admin_add_page_edittext_hint),
                textHint = getString(R.string.group_admin_add_page_text_restriction))


        val categoryAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        categorySpinner.spinner.adapter = categoryAdapter
        val categoryModelData = EditableTextModel(getString(R.string.spinner_category_title))
        categorySpinner.setUpModel(categoryModelData)

        val topicAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val topicModelData = EditableTextModel(getString(R.string.spinner_topic_title))
        topicSpinner.setUpModel(topicModelData)

        if (savedInstanceState != null) {
            isFirstTime = false
            isRotated = true
            if (savedInstanceState.containsKey(CATEGORY_SELECTED_ID) && savedInstanceState.containsKey(CATEGORY_ID)) {
                mCategorySelectedID = savedInstanceState.getInt(CATEGORY_SELECTED_ID)
                mCategoryID = savedInstanceState.getString(CATEGORY_ID)
                categorySpinner.spinner.setSelection(mCategorySelectedID!!)
            }

            if (savedInstanceState.containsKey(TOPIC_SELECTED_ID) && savedInstanceState.containsKey(TOPIC_ID)) {
                mTopicID = savedInstanceState.getString(TOPIC_SELECTED_ID)
                mTopicSelectedID = savedInstanceState.getInt(TOPIC_ID)
            }
        }


        if (isFirstTime) {
            setProperDataFromLaunchActivity(categoryAdapter, topicAdapter, textModelData)
        }

        textInput.setUpModel(textModelData)

        val viewModel = ViewModelProviders.of(this).get(GroupAdminViewModel::class.java)
        viewModel.getAllCategories().observe(this, Observer { data ->
            categoryAdapter.swapData(data as ArrayList<Selectable>)
            if (mCategorySelectedID != null) {
                categorySpinner.spinner.setSelection(mCategorySelectedID!!)
            }
        })

        viewModel.topics.observe(this, Observer { data ->
            topicAdapter.swapData(data as ArrayList<Selectable>)
            topicSpinner.spinner.adapter = topicAdapter
            if (mTopicSelectedID != null) {
                topicSpinner.spinner.setSelection(mTopicSelectedID!!)
            }
        })

        setCategorySpinnerListener(categoryAdapter, viewModel)
        setTopicSpinnerListener(topicAdapter, viewModel)


        buttonAdd.setOnClickListener {
            if (NetworkUtil.isNetworkAvailable(this)) {
                updateGroup(viewModel)
            } else {
                Snacker.show(mainLayout, getString(R.string.group_admin_page_edit_internet_problem))
            }
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun updateGroup(viewModel: GroupAdminViewModel) {
        if (UtilChecker.hasCorrectData(textInput.isModelValid())) {
            launch(UI) {
                progressBar.visibility = View.VISIBLE
                val editedGroup = viewModel.editGroup(mTopicID, mGroupID, textInput.getTextInputData())
                progressBar.visibility = View.INVISIBLE
                val intent = Intent()
                intent.putExtra(ExtraIntents.GROUP_MESSAGE, getString(editedGroup.messageResource))
                setResult(Activity.RESULT_OK, intent)
                finish()
            }
        } else {
            textInput.textInputEditText.error = textInput.hintTextView.text
        }
    }

    private fun setProperDataFromLaunchActivity(categoryAdapter: DefaultSpinnerAdapter, topicAdapter: DefaultSpinnerAdapter, textModel: EditableTextModel) {
        val extra = intent.extras
        if (extra != null) {
            if (extra.containsKey(ExtraIntents.GROUP_EDIT) &&
                    extra.containsKey(ExtraIntents.GROUP_TOPIC_ID) &&
                    extra.containsKey(ExtraIntents.GROUP_CATEGORY_ID) &&
                    extra.containsKey(ExtraIntents.GROUP_TOPIC_SELECTED) &&
                    extra.containsKey(ExtraIntents.GROUP_CATEGORY_SELECTED) &&
                    extra.containsKey(ExtraIntents.GROUP_ID) &&
                    extra.containsKey(ExtraIntents.GROUP_DESCRIPTION)) {

                mTopicID = extra.getString(ExtraIntents.GROUP_TOPIC_ID)
                mCategoryID = extra.getString(ExtraIntents.GROUP_CATEGORY_ID)
                mGroupID = extra.getString(ExtraIntents.GROUP_ID)

                mTopicSelectedID = extra.getInt(ExtraIntents.GROUP_TOPIC_SELECTED)
                mCategorySelectedID = extra.getInt(ExtraIntents.GROUP_CATEGORY_SELECTED)

                textModel.editTextValue = extra.getString(ExtraIntents.GROUP_DESCRIPTION)

            }
        }
    }


    private fun setTopicSpinnerListener(topicAdapter: DefaultSpinnerAdapter, viewModel: GroupAdminViewModel) {
        topicSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mTopicID = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                mTopicSelectedID = position
                mTopicID = topicAdapter.getActualitem(position).getPathID()
                viewModel.topicID.value = mTopicID
            }
        }
    }

    private fun setCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter, viewModel: GroupAdminViewModel) {
        categorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategoryID = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                mCategorySelectedID = position
                mCategoryID = categoryAdapter.getActualitem(position).getPathID()
                viewModel.categoryID.value = mCategoryID
                if (!isRotated) {
                    mTopicID = null
                } else {
                    isRotated = false
                }
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mCategorySelectedID != null && mCategoryID != null) {
            Log.v("Palo", mCategorySelectedID.toString())
            outState?.putInt(CATEGORY_SELECTED_ID, mCategorySelectedID!!)
            outState?.putString(CATEGORY_ID, mCategoryID)
        }

        if (mTopicID != null && mTopicSelectedID != null) {
            outState?.putString(TOPIC_ID, mTopicID)
            outState?.putInt(TOPIC_SELECTED_ID, mTopicSelectedID!!)
        }
    }


    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }


}