package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.Action
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.model.Task
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.task_add.TaskAdminAddActivity
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.task_edit.TaskAdminEditActivity
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_task.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_data.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 20.01.2018.
 */

class TaskAdminActivity : AppCompatActivity(), TaskAdminAdapter.OnClickAdminTaskListener,
        TaskAdminDeleteDialog.OnDialogClick, INetworkChecker {

    val TASK_DIALOG = "task_dialog"

    val CATEGORY_SELECTED_VALUE = "CATEGORY_SELECTED_VALUE"
    val CATEGORY_SELECTED_ID = "CATEGORY_SELECTED_ID"
    val TOPIC_SELECTED_VALUE = "TOPIC_SELECTED_VALUE"
    val TOPIC_SELECTED_ID = "TOPIC_SELECTED_ID"
    val GROUP_SELECTED_VALUE = "GROUP_SELECTED_VALUE"
    val GROUP_SELECTED_ID = "GROUP_SELECTED_ID"
    val EXERCISE_SELECTED_VALUE = "EXERCISE_SELECTED_VALUE"
    val EXERCISE_SELECTED_ID = "EXERCISE_SELECTED_ID"

    var mTaskID: String? = null

    var mCategorySelectedID: Int? = null
    var mCategorySelectedValue: String? = null

    var mTopicSelectedID: Int? = null
    var mTopicSelectedValue: String? = null

    var mGroupSelectedID: Int? = null
    var mGroupSelectedValue: String? = null

    var mExerciseSelectedID: Int? = null
    var mExerciseSelectedValue: String? = null


    var viewModel: TaskAdminViewModel? = null

    override var connectionObserver: ConnectionObserver? = null
    private var isRotatedCategory = false
    private var isRotatedTopic = false
    private var isRotatedGroup = false
    private var isRotatedExercise = false


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_task)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.task_admin_page_task_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)


        if (savedInstanceState != null && savedInstanceState.containsKey(CATEGORY_SELECTED_ID) && savedInstanceState.containsKey(CATEGORY_SELECTED_VALUE)) {
            mCategorySelectedValue = savedInstanceState.getString(CATEGORY_SELECTED_VALUE)
            mCategorySelectedID = savedInstanceState.getInt(CATEGORY_SELECTED_ID)
            isRotatedCategory = true
        }
        if (savedInstanceState != null && savedInstanceState.containsKey(TOPIC_SELECTED_ID) && savedInstanceState.containsKey(TOPIC_SELECTED_VALUE)) {
            mTopicSelectedValue = savedInstanceState.getString(TOPIC_SELECTED_VALUE)
            mTopicSelectedID = savedInstanceState.getInt(TOPIC_SELECTED_ID)
            isRotatedTopic = true
        }

        if (savedInstanceState != null && savedInstanceState.containsKey(GROUP_SELECTED_VALUE) && savedInstanceState.containsKey(GROUP_SELECTED_ID)) {
            mGroupSelectedID = savedInstanceState.getInt(GROUP_SELECTED_ID)
            mGroupSelectedValue = savedInstanceState.getString(GROUP_SELECTED_VALUE)
            isRotatedGroup = true
        }

        if (savedInstanceState != null && savedInstanceState.containsKey(EXERCISE_SELECTED_VALUE) && savedInstanceState.containsKey(EXERCISE_SELECTED_ID)) {
            mExerciseSelectedID = savedInstanceState.getInt(EXERCISE_SELECTED_ID)
            mExerciseSelectedValue = savedInstanceState.getString(EXERCISE_SELECTED_VALUE)
            isRotatedExercise = true
        }


        val categoryAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerCategoryData = EditableTextModel(getString(R.string.spinner_category_title))
        categorySpinner.setUpModel(spinnerCategoryData)

        val topicAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerTopicData = EditableTextModel(getString(R.string.spinner_topic_title))
        topicSpinner.setUpModel(spinnerTopicData)

        val groupAdatper = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerGroupData = EditableTextModel(getString(R.string.spinner_group_title))
        groupSpinner.setUpModel(spinnerGroupData)

        val exerciseAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        val spinnerExerciseData = EditableTextModel(getString(R.string.spinner_exercise_title))
        exerciseSpinner.setUpModel(spinnerExerciseData)

        val taskAdapter = TaskAdminAdapter(this)
        taskAdminRecyclerView.layoutManager = LinearLayoutManager(this)
        taskAdminRecyclerView.adapter = taskAdapter

        categorySpinner.spinner.adapter = categoryAdapter

        viewModel = ViewModelProviders.of(this).get(TaskAdminViewModel::class.java)



        viewModel?.getAllCategories()?.observe(this, Observer { data ->
            categoryAdapter.swapData(data as ArrayList<Selectable>)
            if (mCategorySelectedID != null) {
                categorySpinner.spinner.setSelection(mCategorySelectedID!!)
            }
            loaderProgress.visibility = View.VISIBLE
        })



        viewModel?.topics?.observe(this, Observer { data ->
            topicAdapter.swapData(data as ArrayList<Selectable>)
            topicSpinner.spinner.adapter = topicAdapter
            if (data.isEmpty()) {
                loaderProgress.visibility = View.GONE
                noAvailableData.visibility = View.VISIBLE
                mTopicSelectedID = null
                mTopicSelectedValue = null
            } else {
                noAvailableData.visibility = View.GONE
            }

            if (mTopicSelectedID != null) {
                topicSpinner.spinner.setSelection(mTopicSelectedID!!)
            }
            groupAdatper.clearData()
        })

        viewModel?.groups?.observe(this, Observer { data ->
            groupAdatper.swapData(data as ArrayList<Selectable>)
            groupSpinner.spinner.adapter = groupAdatper
            if (data.isEmpty()) {
                loaderProgress.visibility = View.GONE
                noAvailableData.visibility = View.VISIBLE
                mGroupSelectedID = null
                mGroupSelectedValue = null
            } else {
                noAvailableData.visibility = View.GONE
            }

            if (mGroupSelectedID != null) {
                groupSpinner.spinner.setSelection(mGroupSelectedID!!)
            }
            exerciseAdapter.clearData()
        })

        viewModel?.exercises?.observe(this, Observer { data ->
            exerciseAdapter.swapData(data as ArrayList<Selectable>)
            exerciseSpinner.spinner.adapter = exerciseAdapter

            if (data.isEmpty()) {
                loaderProgress.visibility = View.GONE
                noAvailableData.visibility = View.VISIBLE
                mExerciseSelectedValue = null
                mExerciseSelectedID = null
            } else {
                noAvailableData.visibility = View.GONE
            }

            if (mExerciseSelectedID != null) {
                exerciseSpinner.spinner.setSelection(mExerciseSelectedID!!)
            }

        })

        viewModel?.tasks?.observe(this, Observer { data ->
            if (data != null) {
                taskAdapter.swapData(data)
                if (data.isEmpty()) {
                    noAvailableData.visibility = View.VISIBLE
                    taskAdminRecyclerView.visibility = View.GONE
                } else {
                    noAvailableData.visibility = View.GONE
                    taskAdminRecyclerView.visibility = View.VISIBLE
                }
            }
        })

        setUpCategorySpinnerListener(categoryAdapter)
        setUpSpinnerTopicListener(topicAdapter)
        setUpSpinnerGroupListener(groupAdatper)
        setUpSpinnerExerciseListener(exerciseAdapter)

        addTask.setOnClickListener {
            if (NetworkUtil.isNetworkAvailable(this)) {
                if (mExerciseSelectedValue != null && mTopicSelectedValue != null && mGroupSelectedValue != null && mCategorySelectedValue != null) {
                    createNewTask()
                } else {
                    Snacker.show(mainLayout, getString(R.string.task_admin_page_add_problem))
                }
            } else {
                Snacker.show(mainLayout, getString(R.string.task_admin_page_add_internet_problem))
            }
        }

    }


    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ExtraIntents.TASK_MESSAGE_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                Snacker.show(mainLayout, data?.getStringExtra(ExtraIntents.TASK_MESSAGE).toString())
            }
        }
    }

    private fun createNewTask() {
        val intent = Intent(this, TaskAdminAddActivity::class.java)
        intent.putExtra(ExtraIntents.TASK_ADD, true)
        intent.putExtra(ExtraIntents.TASK_EXERCISE_ID, mExerciseSelectedValue)
        startActivityForResult(intent, ExtraIntents.TASK_MESSAGE_CODE)
    }

    private fun setUpSpinnerExerciseListener(exerciseAdapter: DefaultSpinnerAdapter) {
        exerciseSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mExerciseSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedExercise) {
                    mExerciseSelectedID = position
                    mExerciseSelectedValue = exerciseAdapter.getActualitem(position).getPathID()
                }
                viewModel?.exerciseID?.value = mExerciseSelectedValue
                if (isRotatedExercise) {
                    isRotatedExercise = false
                }
            }
        }
    }

    private fun setUpCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter) {
        categorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategorySelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedCategory) {
                    mCategorySelectedID = position
                    mCategorySelectedValue = categoryAdapter.getActualitem(position).getPathID()
                    mTopicSelectedValue = null
                    mTopicSelectedID = null
                }
                viewModel?.categoryID?.value = mCategorySelectedValue
                if (isRotatedCategory) {
                    isRotatedCategory = false
                }
            }
        }
    }

    private fun setUpSpinnerTopicListener(topicAdapter: DefaultSpinnerAdapter) {
        topicSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onNothingSelected(parent: AdapterView<*>?) {
                mTopicSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedTopic) {
                    mTopicSelectedID = position
                    mTopicSelectedValue = topicAdapter.getActualitem(position).getPathID()
                    mGroupSelectedID = null
                    mGroupSelectedValue = null
                }
                viewModel?.topicID?.value = mTopicSelectedValue
                if (isRotatedTopic) {
                    isRotatedTopic = false
                }
            }
        }
    }

    private fun setUpSpinnerGroupListener(groupAdatper: DefaultSpinnerAdapter) {
        groupSpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mGroupSelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (!isRotatedGroup) {
                    mGroupSelectedID = position
                    mGroupSelectedValue = groupAdatper.getActualitem(position).getPathID()
                    mExerciseSelectedID = null
                    mExerciseSelectedValue = null
                }
                viewModel?.groupID?.value = mGroupSelectedValue
                if (isRotatedGroup) {
                    isRotatedGroup = false
                }
            }
        }
    }


    override fun onClick(task: Task, action: Action) {
        when (action) {
            Action.EDIT -> editTask(task)
            Action.DELETE -> deleteTask(task)
        }
    }

    private fun deleteTask(task: Task) {
        mTaskID = task.taskID
        val dialog = TaskAdminDeleteDialog()
        dialog.show(fragmentManager, TASK_DIALOG)
    }

    override fun onClickDelete() {
        if (NetworkUtil.isNetworkAvailable(this)) {
            launch(UI) {
                val message = viewModel?.deleteTask(mTaskID)
                Snacker.show(mainLayout, getString(message!!.messageResource))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.task_admin_page_delete_internet_problem))
        }
    }


    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mCategorySelectedID != null && mCategorySelectedValue != null) {
            outState?.putInt(CATEGORY_SELECTED_ID, mCategorySelectedID!!)
            outState?.putString(CATEGORY_SELECTED_VALUE, mCategorySelectedValue)
        }

        if (mTopicSelectedValue != null && mTopicSelectedID != null) {
            outState?.putString(TOPIC_SELECTED_VALUE, mTopicSelectedValue)
            outState?.putInt(TOPIC_SELECTED_ID, mTopicSelectedID!!)
        }

        if (mGroupSelectedValue != null && mGroupSelectedID != null) {
            outState?.putString(GROUP_SELECTED_VALUE, mGroupSelectedValue)
            outState?.putInt(GROUP_SELECTED_ID, mGroupSelectedID!!)
        }

        if (mExerciseSelectedValue != null && mExerciseSelectedID != null) {
            outState?.putString(EXERCISE_SELECTED_VALUE, mExerciseSelectedValue)
            outState?.putInt(EXERCISE_SELECTED_ID, mExerciseSelectedID!!)
        }
    }

    private fun editTask(task: Task) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            val intent = Intent(this, TaskAdminEditActivity::class.java)
            intent.putExtra(ExtraIntents.TASK_EDIT, true)
            intent.putExtra(ExtraIntents.TASK_EXERCISE_ID, mExerciseSelectedValue)
            intent.putExtra(ExtraIntents.TASK_ID, task.taskID)
            startActivityForResult(intent, ExtraIntents.TASK_MESSAGE_CODE)
        } else {
            Snacker.show(mainLayout, getString(R.string.task_admin_page_edit_internet_problem))
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }


}