package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.Transformations
import android.arch.lifecycle.ViewModel
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult
import cz.uhk.pavolpodstreleny.schoolproject.data.repositories.TaskAdminRepository
import cz.uhk.pavolpodstreleny.schoolproject.model.*
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.SingleLiveEvent

/**
 * Created by pavol.podstreleny on 21.01.2018.
 */
class TaskAdminViewModel : ViewModel() {


    private val mTaskAdminRepository = TaskAdminRepository()
    private var categories: SingleLiveEvent<List<Category>>? = null

    var taskID: MutableLiveData<String> = MutableLiveData()
    var task: LiveData<Task> = Transformations.switchMap(taskID, { id ->
        getTaskFromId(id)
    })


    var categoryID: MutableLiveData<String> = MutableLiveData()
    val topics: LiveData<List<Topic>> = Transformations.switchMap(categoryID, { id ->
        getAllTopicsInCategory(id)
    })

    var topicID: MutableLiveData<String> = MutableLiveData()
    val groups: LiveData<List<Group>> = Transformations.switchMap(topicID, { id ->
        getAllGroupsInTopic(id)
    })

    var groupID: MutableLiveData<String> = MutableLiveData()
    val exercises: LiveData<List<Exercise>> = Transformations.switchMap(groupID, { id -> getAllExercisesInGroup(id) }
    )

    var exerciseID: MutableLiveData<String> = MutableLiveData()
    val tasks: LiveData<ArrayList<Task>> = Transformations.switchMap(exerciseID, { id ->
        getAllTasksInExercise(id)
    })

    var userID: MutableLiveData<String> = MutableLiveData()
    val userBadges: LiveData<List<String>> = Transformations.switchMap(userID, { id ->
        getAllUserBadges(id)
    })

    var numberOfTries: MutableLiveData<String> = MutableLiveData()
    val number: LiveData<Int> = Transformations.switchMap(numberOfTries, { id ->
        val lister: List<String> = id.split("/").map { it.trim() }
        getUserNumberOfFinished(lister.get(0), lister.get(1))
    })


    private fun getAllUserBadges(userID: String): LiveData<List<String>> {
        return mTaskAdminRepository.getAllUserBadges(userID)
    }

    private fun getUserNumberOfFinished(exerciseID: String, taskID: String): LiveData<Int> {
        return mTaskAdminRepository.getUserNumberOfFinished(exerciseID, taskID)
    }

    private fun getAllTasksInExercise(id: String?): LiveData<ArrayList<Task>>? {
        return mTaskAdminRepository.getAllTasksInExercise(id)
    }

    suspend fun addBadge(userID: String, badgeID: String): FirebaseResult {
        return mTaskAdminRepository.addBadge(userID, badgeID)
    }

    fun getAllBadges(): LiveData<List<Badge>> {
        return mTaskAdminRepository.getAllBadges()
    }

    fun getAllCategories(): SingleLiveEvent<List<Category>> {
        if (categories == null) {
            categories = mTaskAdminRepository.getAllCategoriesForSpinner()
        }
        return categories!!
    }


    fun getUserPoints(userID: String): LiveData<Double> {
        return mTaskAdminRepository.getUserPoints(userID)
    }

    fun getAllTopicsInCategory(id: String): SingleLiveEvent<List<Topic>> {
        return mTaskAdminRepository.getAllTopicsInCategorySpinner(id)
    }

    fun getAllGroupsInTopic(id: String): SingleLiveEvent<List<Group>> {
        return mTaskAdminRepository.getAllGroupsInTopicSpinner(id)
    }

    fun getAllExercisesInGroup(id: String): SingleLiveEvent<List<Exercise>> {
        return mTaskAdminRepository.getAllExercisesInGroupSpinner(id)
    }

    suspend fun deleteTask(taskID: String?): FirebaseResult {
        return mTaskAdminRepository.deleteTask(taskID)
    }

    suspend fun addTask(description: String, exerciseID: String?, options: HashMap<String, Boolean>): FirebaseResult {
        return mTaskAdminRepository.addTask(description, exerciseID, options)
    }

    suspend fun setPoints(points: Double, userID: String, exerciseID: String?, topicID: String, numberOfTries: Int): FirebaseResult {
        return mTaskAdminRepository.setPoints(points, userID, exerciseID, topicID, numberOfTries)
    }

    fun getTaskFromId(id: String): LiveData<Task> {
        return mTaskAdminRepository.getTaskFromId(id)
    }


    suspend fun editTask(taskID: String?, description: String, options: HashMap<String, Boolean>): FirebaseResult {
        return mTaskAdminRepository.editExercise(taskID, description, options)
    }

}