package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.task_add

import android.app.Activity
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.TaskOption
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.TaskAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_task_add.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 21.01.2018.
 */
class TaskAdminAddActivity : AppCompatActivity(), INetworkChecker {

    private val MINIMUM_DESCRIPTION_LENGTH = 10
    private val MINIMUM_CHOICE_LENGTH = 1

    private var mExerciseID: String? = null
    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_task_add)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        setSupportActionBar(toolbar)
        supportActionBar?.title = getString(R.string.task_admin_add_page_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val mTaskOptionAdapter = TaskOptionAdapter()

        val descriptionModel = EditableTextModel(
                hints = getString(R.string.task_admin_add_page_task_description_hint),
                textHint = getString(R.string.task_admin_add_page_task_restriction))
        textInput.setUpModel(descriptionModel)

        val optionsModel = EditableTextModel(
                hints = getString(R.string.task_admin_add_page_task_choice_hint),
                textHint = getString(R.string.task_admin_add_page_task_choice_restriction_more))
        choiceInput.setUpModel(optionsModel)

        choiceRecyclerView.layoutManager = LinearLayoutManager(this)
        choiceRecyclerView.adapter = mTaskOptionAdapter

        textInput.MINIMUM_LENGHT = MINIMUM_DESCRIPTION_LENGTH
        choiceInput.MINIMUM_LENGHT = MINIMUM_CHOICE_LENGTH


        val viewModel = ViewModelProviders.of(this).get(TaskAdminViewModel::class.java)

        choiceAddButton.setOnClickListener {
            if (choiceInput.isModelValid()) {

                val choices = choiceInput.getTextInputData()

                if (!choices.contains(".") && !choices.contains("/") && !choices.contains("#") && !choices.contains("$")
                        && !choices.contains("[") && !choices.contains("]")
                        ) {

                    val option = TaskOption(choiceInput.getTextInputData(), false)


                    mTaskOptionAdapter.addData(option)
                    choiceInput.clearEditData()
                } else {
                    choiceInput.textInputEditText.error = getString(R.string.task_admin_add_page_task_choice_restriction_error)
                }
            } else {
                choiceInput.textInputEditText.error = getString(R.string.task_admin_add_page_task_choice_restriction)
            }
        }

        buttonAdd.setOnClickListener {
            if (textInput.isModelValid()) {
                if (mTaskOptionAdapter.getStringOptions().isEmpty()) {
                    Snacker.show(mainLayout, getString(R.string.task_admin_page_add_answer_problem))
                    return@setOnClickListener
                }

                if (NetworkUtil.isNetworkAvailable(this)) {
                    launch(UI) {
                        progressBar.visibility = View.VISIBLE

                        val addMessage = viewModel.addTask(textInput.getTextInputData(), mExerciseID, mTaskOptionAdapter.getStringOptions())

                        progressBar.visibility = View.INVISIBLE

                        val intent = Intent()
                        intent.putExtra(ExtraIntents.TASK_MESSAGE, getString(addMessage.messageResource))
                        setResult(Activity.RESULT_OK, intent)
                        finish()
                    }
                } else {
                    Snacker.show(mainLayout, getString(R.string.task_admin_page_add_internet_problem))
                }
            } else {
                textInput.textInputEditText.error = getString(R.string.task_admin_add_page_task_restriction)
            }
        }

        getExtra()

    }

    private fun getExtra() {
        val extra = intent.extras
        if (extra != null && extra.containsKey(ExtraIntents.TASK_EXERCISE_ID)) {
            mExerciseID = extra.getString(ExtraIntents.TASK_EXERCISE_ID)
        }
    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }


    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

}