package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.task_edit

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.TaskOption
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.TaskAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.task_add.TaskOptionAdapter
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_task_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch


/**
 * Created by pavol.podstreleny on 21.01.2018.
 */
class TaskAdminEditActivity : AppCompatActivity(), INetworkChecker {

    private var mTaskID: String? = null
    private var mExerciseID: String? = null

    override var connectionObserver: ConnectionObserver? = null
    private val MINIMUM_DESCRIPTION_LENGTH = 10
    private val MINIMUM_CHOICE_LENGTH = 1

    private var isFirstRun = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_task_edit)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet),this,this)

        setSupportActionBar(toolbar)
        supportActionBar?.title = getString(R.string.task_admin_edit_page_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        buttonAdd.text = getString(R.string.global_button_edit)

        if(savedInstanceState != null){
            isFirstRun = false
        }

        if(isFirstRun){
            isFirstRun = false
            getExtra()
        }

        val mTaskOptionAdapter = TaskOptionAdapter()

        val descriptionModel = EditableTextModel(
                hints = getString(R.string.task_admin_add_page_task_description_hint),
                textHint = getString(R.string.task_admin_add_page_task_restriction))
        textInput.setUpModel(descriptionModel)

        val optionsModel = EditableTextModel(
                hints = getString(R.string.task_admin_add_page_task_choice_hint),
                textHint = getString(R.string.task_admin_add_page_task_choice_restriction))
        choiceInput.setUpModel(optionsModel)

        choiceRecyclerView.layoutManager = LinearLayoutManager(this)
        choiceRecyclerView.adapter = mTaskOptionAdapter

        textInput.MINIMUM_LENGHT = MINIMUM_DESCRIPTION_LENGTH
        choiceInput.MINIMUM_LENGHT = MINIMUM_CHOICE_LENGTH

        val viewModel = ViewModelProviders.of(this).get(TaskAdminViewModel::class.java)

        if(mTaskID != null){
            viewModel.taskID.value = mTaskID
        }

        viewModel.task.observe(this, Observer {
            data ->
             if(data != null){
                 descriptionModel.editTextValue = data.description
                 textInput.setUpModel(descriptionModel)

                 data.choices.forEach {
                     map -> mTaskOptionAdapter.addData(TaskOption(map.key.toString(),map.value))
                 }
             }
        })


        choiceAddButton.setOnClickListener {
            if(choiceInput.isModelValid()){

                val choices = choiceInput.getTextInputData()

                if(!choices.contains(".") && !choices.contains("/") && !choices.contains("#") && !choices.contains("$")
                        && !choices.contains("[") && !choices.contains("]")
                        ) {

                    val option = TaskOption(choiceInput.getTextInputData(), false)


                    mTaskOptionAdapter.addData(option)
                    choiceInput.clearEditData()
                }else{
                    choiceInput.textInputEditText.error = getString(R.string.task_admin_add_page_task_choice_restriction_error)
                }

            }else{
                choiceInput.textInputEditText.error = getString(R.string.task_admin_add_page_task_choice_restriction)
            }
        }

        buttonAdd.setOnClickListener{
            if(NetworkUtil.isNetworkAvailable(this)) {
                if (textInput.isModelValid()) {
                    launch(UI) {

                        progressBar.visibility = View.VISIBLE

                        val addMessage = viewModel.editTask(mTaskID, textInput.getTextInputData(), mTaskOptionAdapter.getStringOptions())

                        progressBar.visibility = View.INVISIBLE

                        val intent = Intent()
                        intent.putExtra(ExtraIntents.TASK_MESSAGE, getString(addMessage.messageResource))
                        setResult(Activity.RESULT_OK, intent)
                        finish()
                    }
                } else {
                    textInput.textInputEditText.error = getString(R.string.task_admin_add_page_task_restriction)
                }
            }else{
                Snacker.show(mainLayout,getString(R.string.task_admin_page_edit_internet_problem))
            }
        }


    }

    private fun getExtra() {
        val extra = intent.extras
        if(extra != null && extra.containsKey(ExtraIntents.TASK_EXERCISE_ID)
                && extra.containsKey(ExtraIntents.TASK_ID)
                && extra.containsKey(ExtraIntents.TASK_EDIT)){
            mExerciseID = extra.getString(ExtraIntents.TASK_EXERCISE_ID)
            mTaskID = extra.getString(ExtraIntents.TASK_ID)
        }
    }


    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

}