package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.Action
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.model.Topic
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.topic_add.TopicAddActivity
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.topic_edit.TopicEditActivity
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_topic.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_data.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch


/**
 * Created by pavol.podstreleny on 09.01.2018.
 */
class TopicAdminActivity : AppCompatActivity(), TopicAdminAdapter.OnClickAdminTopicListener, TopicAdminDeleteDialog.OnDialogClick, INetworkChecker {

    val TOPIC_DIALOG = "topic_dialog"
    val CATEGORY_SELECTED_ID = "category_selected_id"
    val CATEGORY_SELECTED_VALUE = "category_selected_value"
    val UNKNOWN = "unknown"

    var mTopicID: String? = null
    var mTopicURL: Uri? = null

    var mCategorySelectedID: Int? = null
    var mCategorySelectedValue: String? = null

    var viewModel: TopicAdminViewModel? = null

    override var connectionObserver: ConnectionObserver? = null

    var isRotated = false


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_topic)

        setSupportActionBar(toolbar)
        supportActionBar?.title = resources.getString(R.string.topic_admin_page_topic_title)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        val categoryAdapter = DefaultSpinnerAdapter(this, arrayListOf())

        val topicAdapter = TopicAdminAdapter(this)
        topicAdminRecyclerView.adapter = topicAdapter
        topicAdminRecyclerView.layoutManager = LinearLayoutManager(this)

        val spinnerData = EditableTextModel(resources.getString(R.string.spinner_category_title))
        categorySpinner.setUpModel(spinnerData)

        setUpCategorySpinnerListener(categoryAdapter)
        categorySpinner.spinner.adapter = categoryAdapter

        viewModel = ViewModelProviders.of(this).get(TopicAdminViewModel::class.java)

        viewModel?.getAllCategories()?.observe(this, Observer { data ->
            categoryAdapter.swapData(data as ArrayList<Selectable>)
            if (isRotated) {
                if (mCategorySelectedID != null) {
                    categorySpinner.spinner.setSelection(mCategorySelectedID!!)
                }
                isRotated = false
            }
            loaderProgress.visibility = View.VISIBLE
        })

        viewModel?.topics?.observe(this, Observer { data ->
            if (data != null) {
                loaderProgress.visibility = View.GONE
                topicAdapter.swapData(data)
                if (data.size != 0) {
                    noAvailableData.visibility = View.GONE
                    topicAdminRecyclerView.visibility = View.VISIBLE
                } else {
                    noAvailableData.visibility = View.VISIBLE
                    topicAdminRecyclerView.visibility = View.GONE
                }
            }
        })


        addTopic.setOnClickListener {
            startAddTopicActivity()
        }


        if (savedInstanceState != null && savedInstanceState.containsKey(CATEGORY_SELECTED_ID) && savedInstanceState.containsKey(CATEGORY_SELECTED_VALUE)) {
            mCategorySelectedValue = savedInstanceState.getString(CATEGORY_SELECTED_VALUE)
            mCategorySelectedID = savedInstanceState.getInt(CATEGORY_SELECTED_ID)

            isRotated = true

        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun startAddTopicActivity() {
        if (mCategorySelectedValue == null) {
            Snacker.show(mainLayout, getString(R.string.topic_admin_page_add_problem))
        } else if (NetworkUtil.isNetworkAvailable(this)) {
            Snacker.show(mainLayout, getString(R.string.topic_admin_page_add_internet_problem))
        } else {
            val intent = Intent(this, TopicAddActivity::class.java)
            intent.putExtra(ExtraIntents.TOPIC_ADD, true)
            intent.putExtra(ExtraIntents.TOPIC_CATEGORY_ID, mCategorySelectedValue)
            startActivityForResult(intent, ExtraIntents.TOPIC_MESSAGE_CODE)
        }
    }

    private fun setUpCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter) {
        categorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategorySelectedValue = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                mCategorySelectedID = position
                mCategorySelectedValue = categoryAdapter.getActualitem(position).getPathID()
                viewModel?.topicID?.value = mCategorySelectedValue
            }
        }
    }

    override fun onPause() {
        super.onPause()
        loaderProgress.visibility = View.GONE
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mCategorySelectedID != null && mCategorySelectedValue != null) {
            outState?.putInt(CATEGORY_SELECTED_ID, mCategorySelectedID!!)
            outState?.putString(CATEGORY_SELECTED_VALUE, mCategorySelectedValue)
        }
    }

    override fun onClick(topic: Topic, action: Action) {
        when (action) {
            Action.EDIT -> startEditActivity(topic)
            Action.DELETE -> deleteActualTopic(topic)
        }
    }

    override fun onClick() {
        if (mTopicID != null && mTopicURL != null && viewModel != null) {
            launch(UI) {
                val deleteMessage = viewModel!!.deleteTopic(mTopicID!!, mTopicURL!!)
                Snacker.show(mainLayout, getString(deleteMessage.messageResource))
            }
        }
    }

    private fun startEditActivity(topic: Topic) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (topic.topicKey != UNKNOWN) {
                val intent = Intent(this, TopicEditActivity::class.java)
                intent.putExtra(ExtraIntents.TOPIC_EDIT, true)
                intent.putExtra(ExtraIntents.TOPIC_EDIT_DESCRIPTION, topic.description)
                intent.putExtra(ExtraIntents.TOPIC_EDIT_PHOTO_URL, topic.photoURL)
                intent.putExtra(ExtraIntents.TOPIC_EDIT_CATEGORY_ID, topic.category)
                intent.putExtra(ExtraIntents.TOPIC_EDIT_TOPIC_ID, topic.topicKey)
                intent.putExtra(ExtraIntents.TOPIC_EDIT_SELECTED_CATEGORY_ID, categorySpinner.spinner.selectedItemPosition)
                startActivityForResult(intent, ExtraIntents.TOPIC_MESSAGE_CODE)
            } else {
                Snacker.show(mainLayout, getString(R.string.topic_admin_page_unknown_edit_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.topic_admin_page_edit_internet_problem))
        }

    }

    private fun deleteActualTopic(topic: Topic) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            if (topic.topicKey != UNKNOWN) {
                mTopicID = topic.topicKey
                mTopicURL = Uri.parse(topic.photoURL)
                TopicAdminDeleteDialog().show(supportFragmentManager, TOPIC_DIALOG)
            } else {
                Snacker.show(mainLayout, getString(R.string.topic_admin_page_unknown_delete_restriction))
            }
        } else {
            Snacker.show(mainLayout, getString(R.string.topic_admin_page_delete_internet_problem))
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ExtraIntents.TOPIC_MESSAGE_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                Snacker.show(mainLayout, data?.getStringExtra(ExtraIntents.TOPIC_MESSAGE).toString())
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }


}
