package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.Transformations
import android.arch.lifecycle.ViewModel
import android.net.Uri
import cz.uhk.pavolpodstreleny.schoolproject.data.FirebaseResult
import cz.uhk.pavolpodstreleny.schoolproject.data.repositories.TopicAdminRepository
import cz.uhk.pavolpodstreleny.schoolproject.model.Category
import cz.uhk.pavolpodstreleny.schoolproject.model.Topic
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.SingleLiveEvent

/**
 * Created by pavol.podstreleny on 10.01.2018.
 */
class TopicAdminViewModel : ViewModel() {

    private var categories: SingleLiveEvent<List<Category>>? = null

    var topicID: MutableLiveData<String> = MutableLiveData()
    val topics: LiveData<ArrayList<Topic>> = Transformations.switchMap(topicID, { id -> getAllTopicsInCategory(id) })


    private val mTopicAdminRepository = TopicAdminRepository()

    fun getAllCategories(): SingleLiveEvent<List<Category>> {
        if (categories == null) {
            categories = mTopicAdminRepository.getAllCategoriesForSpinner()
        }
        return categories!!
    }

    private fun getAllTopicsInCategory(id: String): LiveData<ArrayList<Topic>> {
        return mTopicAdminRepository.getAllTopicsInCategory(id)
    }

    suspend fun addTopic(description: String, photoURL: Uri?, categoryID: String?): FirebaseResult {
        return mTopicAdminRepository.addTopic(description, photoURL, categoryID)
    }

    suspend fun deleteTopic(topicID: String, photoURL: Uri): FirebaseResult {
        return mTopicAdminRepository.deleteTopic(topicID, photoURL)
    }

    suspend fun editTopic(newImageUrl: Uri?, oldImageUri: Uri?, categoryID: String?, topicID: String?, description: String): FirebaseResult {
        return mTopicAdminRepository.editTopic(newImageUrl, oldImageUri, categoryID, topicID, description)
    }


}