package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.topic_add

import android.app.Activity
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import com.bumptech.glide.Glide
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.TopicAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_category_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.android.synthetic.main.view_upload_button.*
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 10.01.2018.
 */
class TopicAddActivity : AppCompatActivity(), INetworkChecker {

    private val URI_UPLOADED_IMAGE = "uri_uploaded_image"

    private var mImageUri: Uri? = null

    private val INTENT_TYPE = "image/png"
    private val RC_PHOTO_PICKER = 13

    private var mUserCategoryID: String? = null
    override var connectionObserver: ConnectionObserver? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_category_edit)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        if (savedInstanceState != null && savedInstanceState.containsKey(URI_UPLOADED_IMAGE)) {
            mImageUri = Uri.parse(savedInstanceState.getString(URI_UPLOADED_IMAGE))
            showImage(mImageUri!!)
        }

        val extras = intent.extras
        if (extras != null) {
            if (extras.containsKey(ExtraIntents.TOPIC_ADD) && extras.containsKey(ExtraIntents.TOPIC_CATEGORY_ID)) {
                supportActionBar?.title = getString(R.string.topic_admin_add_page_title)
                mUserCategoryID = extras.getString(ExtraIntents.TOPIC_CATEGORY_ID)
            }
        }

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)


        setSupportActionBar(toolbar)
        supportActionBar?.title = getString(R.string.topic_admin_add_page_title)

        val editTextData = EditableTextModel(
                hints = getString(R.string.topic_admin_add_page_edittext_hint),
                textHint = getString(R.string.topic_admin_add_page_text_restriction))
        textInput.setUpModel(editTextData)

        val viewModel = ViewModelProviders.of(this).get(TopicAdminViewModel::class.java)

        buttonUpload.setOnClickListener {
            startImagePicker()
        }

        buttonAdd.setOnClickListener {
            if (NetworkUtil.isNetworkAvailable(this)) {
                createTopic(viewModel)
            } else {
                Snacker.show(mainLayout, getString(R.string.topic_admin_page_add_internet_problem))
            }
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }


    private fun createTopic(viewModel: TopicAdminViewModel) {
        var topicPassed = false
        var imagePassed = false


        if (textInput.isModelValid()) {
            topicPassed = true
        } else {
            textInput.textInputEditText.error = getString(R.string.topic_admin_add_page_text_restriction)
            return
        }

        if (mImageUri != null) {
            imagePassed = true
        }

        if (imagePassed && topicPassed) {
            var message = ""
            progressBar.visibility = View.VISIBLE
            launch {
                val addTopic = viewModel.addTopic(textInput.getTextInputData(), mImageUri, mUserCategoryID)
                message = getString(addTopic.messageResource)

                val intent = Intent()
                intent.putExtra(ExtraIntents.TOPIC_MESSAGE, message)
                setResult(Activity.RESULT_OK, intent)
                finish()
            }
        }
    }

    private fun startImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = INTENT_TYPE
        intent.putExtra(Intent.EXTRA_LOCAL_ONLY, true)
        startActivityForResult(Intent.createChooser(intent, getString(R.string.category_admin_add_page_picker_title)), RC_PHOTO_PICKER)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == RC_PHOTO_PICKER && resultCode == RESULT_OK) {
            mImageUri = data?.data
            if (mImageUri != null) {
                showImage(mImageUri!!)
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun showImage(uri: Uri) {
        uploadedImageView.visibility = View.VISIBLE
        Glide.with(this).load(uri).into(uploadedImageView)
    }

    override fun onStop() {
        super.onStop()
        progressBar.visibility = View.GONE
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mImageUri != null) {
            outState?.putString(URI_UPLOADED_IMAGE, mImageUri.toString())
        }
    }


}