package cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.topic_edit

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.text.Editable
import android.view.MenuItem
import android.view.View
import android.widget.AdapterView
import com.bumptech.glide.Glide
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.custom_views.view_model.EditableTextModel
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Selectable
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.DefaultSpinnerAdapter
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.TopicAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_admin_topic_edit.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino.*
import kotlinx.android.synthetic.main.view_submit_button.*
import kotlinx.android.synthetic.main.view_upload_button.*
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 12.01.2018.
 */
class TopicEditActivity : AppCompatActivity(), INetworkChecker {

    private val RC_PHOTO_PICKER = 14
    private val IS_UPLOADED_NEW = "is_uploaded_new"
    private val IMAGE_TOPIC = "image_topic"
    private val INTENT_TYPE = "image/png"
    private val CATEGORY_ID = "category_id"


    private var mCategoryID: Int? = null
    private var mCategorySelectedValue: String? = null

    private var mOldPhotoURL: Uri? = null
    private var mTopicPhotoURL: Uri? = null
    private var mTopicID: String? = null

    private var isRotated = false
    override var connectionObserver: ConnectionObserver? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_admin_topic_edit)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        toolbar.title = getString(R.string.topic_admin_edit_page_title)
        buttonAdd.text = getString(R.string.global_button_edit)


        val categorySpinnerAdapter = DefaultSpinnerAdapter(this, arrayListOf())
        setCategorySpinner(categorySpinnerAdapter)

        setTopicEditText()

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)


        val viewModel = ViewModelProviders.of(this).get(TopicAdminViewModel::class.java)

        viewModel.getAllCategories().observe(this, Observer { data ->
            categorySpinnerAdapter.swapData(data as ArrayList<Selectable>)
            if (isRotated) {
                topicCategorySpinner.spinner.setSelection(mCategoryID!!)
                isRotated = false
            }

        })


        buttonUpload.setOnClickListener {
            startImagePicker()
        }

        buttonAdd.setOnClickListener {
            if (NetworkUtil.isNetworkAvailable(this)) {
                editCategory(viewModel)
            } else {
                Snacker.show(mainLayout, getString(R.string.topic_admin_page_edit_internet_problem))
            }
        }

        setCategorySpinnerListener(categorySpinnerAdapter)

        val extra = intent.extras

        setExtraData(extra)
        setSaveInstanceData(savedInstanceState)


    }


    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun setSaveInstanceData(savedInstanceState: Bundle?) {
        if (savedInstanceState != null &&
                savedInstanceState.containsKey(IMAGE_TOPIC) &&
                savedInstanceState.containsKey(IS_UPLOADED_NEW) &&
                savedInstanceState.containsKey(CATEGORY_ID)
                ) {
            if (savedInstanceState.getBoolean(IS_UPLOADED_NEW)) {
                mTopicPhotoURL = Uri.parse(savedInstanceState.getString(IMAGE_TOPIC))
                showImage(mTopicPhotoURL)
            } else {
                mOldPhotoURL = Uri.parse(savedInstanceState.getString(IMAGE_TOPIC))
                showImage(mOldPhotoURL)
            }
            mCategoryID = savedInstanceState.getInt(CATEGORY_ID)
            isRotated = true
        }
    }


    private fun setCategorySpinnerListener(categoryAdapter: DefaultSpinnerAdapter) {
        topicCategorySpinner.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {

            override fun onNothingSelected(parent: AdapterView<*>?) {
                mCategoryID = null
            }

            override fun onItemSelected(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                mCategoryID = position
                mCategorySelectedValue = categoryAdapter.getActualitem(position).getPathID()
            }
        }
    }


    private fun setExtraData(extra: Bundle?) {
        if (extra != null) {
            if (extra.containsKey(ExtraIntents.TOPIC_EDIT_DESCRIPTION) &&
                    extra.containsKey(ExtraIntents.TOPIC_EDIT_PHOTO_URL) &&
                    extra.containsKey(ExtraIntents.TOPIC_EDIT_CATEGORY_ID) &&
                    extra.containsKey(ExtraIntents.TOPIC_EDIT_TOPIC_ID) &&
                    extra.containsKey(ExtraIntents.TOPIC_EDIT_SELECTED_CATEGORY_ID)) {


                textInput.textInputEditText.text = Editable.Factory.getInstance().newEditable(extra.getString(ExtraIntents.TOPIC_EDIT_DESCRIPTION))
                mCategorySelectedValue = extra.getString(ExtraIntents.TOPIC_EDIT_CATEGORY_ID)
                mOldPhotoURL = Uri.parse(extra.getString(ExtraIntents.TOPIC_EDIT_PHOTO_URL))
                mTopicID = extra.getString(ExtraIntents.TOPIC_EDIT_TOPIC_ID)
                mCategoryID = extra.getInt(ExtraIntents.TOPIC_EDIT_SELECTED_CATEGORY_ID)


                isRotated = true

                if (mTopicPhotoURL == null) {
                    showImage(mOldPhotoURL)
                }
            }
        }

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == RC_PHOTO_PICKER && resultCode == RESULT_OK) {
            mTopicPhotoURL = data?.data
            if (mTopicPhotoURL != null) {
                showImage(mTopicPhotoURL)
            }
        }
    }

    private fun startImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = INTENT_TYPE
        intent.putExtra(Intent.EXTRA_LOCAL_ONLY, true)
        startActivityForResult(Intent.createChooser(intent, getString(R.string.category_admin_add_page_picker_title)), RC_PHOTO_PICKER)
    }

    private fun setCategorySpinner(adapter: DefaultSpinnerAdapter) {
        val categorySpinnerData = EditableTextModel(getString(R.string.spinner_category_title))
        topicCategorySpinner.setUpModel(categorySpinnerData)
        topicCategorySpinner.spinner.adapter = adapter
    }

    private fun setTopicEditText() {
        val topicEditableData = EditableTextModel(
                hints = getString(R.string.topic_admin_add_page_edittext_hint),
                textHint = getString(R.string.topic_admin_add_page_text_restriction)
        )
        textInput.setUpModel(topicEditableData)
    }

    private fun showImage(uri: Uri?) {
        uploadedImageView.visibility = View.VISIBLE
        Glide.with(this).load(uri).into(uploadedImageView)
    }

    private fun editCategory(viewModel: TopicAdminViewModel) {
        var topicPassed = false
        var imagePassed = false

        if (textInput.isModelValid()) {
            topicPassed = true
        } else {
            textInput.textInputEditText.error = getString(R.string.topic_admin_add_page_text_restriction)
            return
        }

        if (mTopicPhotoURL != null || mOldPhotoURL != null) {
            imagePassed = true
        }

        if (imagePassed && topicPassed) {
            var message = ""
            progressBar.visibility = View.VISIBLE
            launch {
                val editTopic = viewModel.editTopic(mTopicPhotoURL, mOldPhotoURL, mCategorySelectedValue, mTopicID, textInput.getTextInputData())
                message = getString(editTopic.messageResource)

                val intent = Intent()
                intent.putExtra(ExtraIntents.TOPIC_MESSAGE, message)
                setResult(Activity.RESULT_OK, intent)
                finish()
            }
        }

    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }


    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        if (mTopicPhotoURL == null) {
            outState?.putString(IMAGE_TOPIC, mOldPhotoURL.toString())
            outState?.putBoolean(IS_UPLOADED_NEW, false)
        } else {
            outState?.putString(IMAGE_TOPIC, mTopicPhotoURL.toString())
            outState?.putBoolean(IS_UPLOADED_NEW, true)
        }
        if (mCategoryID != null) {
            outState?.putInt(CATEGORY_ID, mCategoryID!!)
        }
    }
}