package cz.uhk.pavolpodstreleny.schoolproject.screens.exercises

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import com.bumptech.glide.Glide
import com.google.firebase.auth.FirebaseAuth
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.UserActivities
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Exercise
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.exercise.ExerciseAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.screens.tasks.TaskActivity
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_topic.*
import kotlinx.android.synthetic.main.view_general_progressbar.*
import kotlinx.android.synthetic.main.view_no_network_connectino_small.*

/**
 * Created by pavol.podstreleny on 10.02.2018.
 */
class ExerciseActivity : AppCompatActivity(), ExerciseAdapter.OnClickExerciseListener, UserActivities, INetworkChecker {

    private var groupName: String? = null
    private var groupID: String? = null

    override var connectionObserver: ConnectionObserver? = null

    private var mAdapter: ExerciseAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_topic)
        setUserActionBar()
        setIntentData()

        val id = FirebaseAuth.getInstance().currentUser?.uid

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        // Set RecyclerView
        mAdapter = ExerciseAdapter(this)
        topicRecycler.layoutManager = LinearLayoutManager(this)
        topicRecycler.adapter = mAdapter

        // GetViewModel and observe topics data
        val viewModel = ViewModelProviders.of(this).get(ExerciseAdminViewModel::class.java)
        viewModel.groupID.value = groupID
        viewModel.exercises.observe(this, Observer { data ->
            generalProgressBar.visibility = View.GONE
            if (data != null) {
                if (data.size > 0) {
                    hideNoDataText()
                    mAdapter?.swapData(data)
                } else {
                    showNoDataText()
                }

            }
        })

        viewModel.user.observe(this, Observer { datas ->
            if (datas != null) {
                if (datas.exercisess != null) {
                    mAdapter?.swapUserExerciseData(datas.exercisess)
                }
            }
        })

        if (id != null) {
            viewModel.userID.value = id
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onClick(exercise: Exercise) {
        if (NetworkUtil.isNetworkAvailable(this)) {
            mAdapter?.exerciseData?.clear()
            mAdapter?.data?.clear()

            val intent = Intent(this, TaskActivity::class.java)
            intent.putExtra(ExtraIntents.EXERCISE_ID, exercise.exerciseKey)
            intent.putExtra(ExtraIntents.EXERCISE_NAME, exercise.description)
            startActivity(intent)
        } else {
            Snacker.show(coordinatorLayout, getString(R.string.exercise_no_internet_no_task))
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == Activity.RESULT_OK) {
            if (data != null) {
                val message = data.getStringExtra(ExtraIntents.USER_MESSAGE)
                Snacker.show(coordinatorLayout, message)
            }
        }
    }

    override fun setIntentData() {
        val extra = intent
        if (extra != null &&
                extra.hasExtra(ExtraIntents.GROUP_ID) &&
                extra.hasExtra(ExtraIntents.GROUP_NAME) &&
                extra.hasExtra(ExtraIntents.TOPIC_NAME)
                ) {

            // Set image/text/id of category
            previousText.text = extra.getStringExtra(ExtraIntents.TOPIC_NAME)
            groupName = extra.getStringExtra(ExtraIntents.GROUP_NAME)
            title = groupName
            groupID = extra.getStringExtra(ExtraIntents.GROUP_ID)
            Glide.with(this).load(extra.getStringExtra(ExtraIntents.TOPIC_URL_IMAGE)).into(ivCategory)
        }
    }

    private fun showNoDataText() {
        noDataLayout.visibility = View.VISIBLE
        noDataText.text = getString(R.string.exercise_page_no_data)
    }

    private fun hideNoDataText() {
        noDataLayout.visibility = View.GONE
    }

    override fun setUserActionBar() {
        // Set actionBar
        setSupportActionBar(specialToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }
}