package cz.uhk.pavolpodstreleny.schoolproject.screens.exercises

import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.model.Exercise
import cz.uhk.pavolpodstreleny.schoolproject.utils.StarUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.roundTo2DecimalPlaces
import java.util.*

/**
 * Created by pavol.podstreleny on 10.02.2018.
 */
class ExerciseAdapter : RecyclerView.Adapter<ExerciseAdapter.BasicViewHolder> {

    var data: ArrayList<Exercise>? = null
    var exerciseData: HashMap<String, List<HashMap<String, Long>>>? = null
    val listener: OnClickExerciseListener

    constructor(listener: OnClickExerciseListener) {
        this.listener = listener
    }

    override fun onCreateViewHolder(parent: ViewGroup?, viewType: Int): BasicViewHolder {
        val view = LayoutInflater.from(parent?.context).inflate(R.layout.list_item_exercise, parent, false)
        return BasicViewHolder(view)
    }

    override fun getItemCount(): Int {
        if (data == null) {
            return 0
        }
        return data!!.size
    }

    override fun onBindViewHolder(holder: BasicViewHolder?, position: Int) {
        holder?.bind(position)
    }

    fun swapUserExerciseData(map: HashMap<String, List<HashMap<String, Long>>>) {
        exerciseData = map
        notifyDataSetChanged()
        if (data == null) {
            return
        } else {
            calculateDifference()
        }
    }

    private fun calculateDifference() {
        if (data != null && exerciseData != null) {
            data!!.forEach { exercise ->
                if (exerciseData!!.containsKey(exercise.exerciseKey)) {
                    val listOfData = exerciseData!!.get(exercise.exerciseKey)
                    listOfData?.forEach { mapa ->
                        exercise.taskMap.forEach({ mapa2 ->
                            if (mapa.containsKey(mapa2.key)) {
                                exercise.userDone++
                            }
                        })

                    }
                }
            }
        }

        notifyDataSetChanged()
    }

    fun swapData(data: ArrayList<Exercise>?) {
        this.data = data
        notifyDataSetChanged()
        if (exerciseData == null) {
            return
        } else {
            calculateDifference()
        }
    }

    inner class BasicViewHolder : RecyclerView.ViewHolder {

        private val layout: LinearLayout
        private val imageView: ImageView
        private val textView: TextView
        private val percentageTextView: TextView

        constructor(view: View) : super(view) {
            this.percentageTextView = view.findViewById(R.id.percentage)
            this.layout = view.findViewById(R.id.generalLayout)
            this.textView = view.findViewById(R.id.generalText)
            this.imageView = view.findViewById(R.id.generalImage)

            layout.setOnClickListener {
                val exercise = data?.get(adapterPosition)
                if (exercise != null) {
                    listener.onClick(exercise)
                }
            }
        }

        fun bind(position: Int) {
            val topic = data?.get(position)
            if (topic != null) {
                textView.text = topic.description

                if (topic.taskMap.size == 0) {
                    percentageTextView.text = ""
                    imageView.setImageResource(StarUtil.showProperStar(-1.0))
                } else {
                    var percentage = 100 * (topic.userDone / topic.taskMap.size.toDouble())
                    if (topic.userDone == 0) {
                        imageView.setImageResource(StarUtil.showProperStar(0.0))
                        percentage = 0.0
                    } else {
                        imageView.setImageResource(StarUtil.showProperStar(percentage))
                    }
                    percentageTextView.text = "${percentage.roundTo2DecimalPlaces()} %"
                }
                topic.userDone = 0
            }
        }
    }

    interface OnClickExerciseListener {
        fun onClick(exercise: Exercise)
    }


}