package cz.uhk.pavolpodstreleny.schoolproject.screens.explore

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.widget.LinearLayoutManager
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Category
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.AdminActivity
import cz.uhk.pavolpodstreleny.schoolproject.screens.topics.TopicActivity
import kotlinx.android.synthetic.main.fragment_explore.*

/**
 * Created by pavol.podstreleny on 22.12.2017.
 */
class ExploreFragment : Fragment(), CategoryAdapter.OnCategoryClickListener {

    val TAG = "ExploreFragment"

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_explore, container, false)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        val mAdapter = CategoryAdapter(context, this)
        categoryRecyclerView.layoutManager = LinearLayoutManager(context)
        categoryRecyclerView.adapter = mAdapter

        val viewModel = ViewModelProviders.of(this).get(CategoryViewModel::class.java)

        viewModel.getAllCategories().observe(this, Observer { data ->
            updateCategoryUI(mAdapter, data)
        })

        viewModel.isUserAdmin()?.observe(this, Observer { data ->
            updateAddIcon(data)
        })

        adminMode.setOnClickListener {
            val intent = Intent(this.context, AdminActivity::class.java)
            startActivity(intent)
        }

    }

    private fun updateCategoryUI(adapter: CategoryAdapter, data: ArrayList<Category>?) {
        if (data != null) {
            adapter.swapData(data)
            exploreProgress.visibility = View.GONE
        }
    }

    private fun updateAddIcon(data: Boolean?) {
        when (data) {
            true -> adminMode.visibility = View.VISIBLE
            false -> adminMode.visibility = View.GONE
            else -> Log.e(TAG, "Null value!")
        }
    }

    override fun onClick(category: Category?) {
        if (category != null) {
            val intent = Intent(activity, TopicActivity::class.java)
            intent.putExtra(ExtraIntents.CATEGORY_URL_IMAGE, category.photoURL)
            intent.putExtra(ExtraIntents.CATEGORY_NAME, category.description)
            intent.putExtra(ExtraIntents.CATEGORY_ID, category.categoryKey)
            startActivity(intent)
        }
    }
}