package cz.uhk.pavolpodstreleny.schoolproject.screens.groups

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import com.bumptech.glide.Glide
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.UserActivities
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Group
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.groups.GroupAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.screens.exercises.ExerciseActivity
import kotlinx.android.synthetic.main.activity_topic.*
import kotlinx.android.synthetic.main.view_general_progressbar.*
import kotlinx.android.synthetic.main.view_no_network_connectino_small.*

/**
 * Created by pavol.podstreleny on 10.02.2018.
 */
class GroupActivity : AppCompatActivity(), GroupAdapter.OnClickGroupListener, UserActivities, INetworkChecker {

    var topicID: String? = null
    var topicName: String? = null

    override var connectionObserver: ConnectionObserver? = null

    private var mTopicUrlImage: String? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_topic)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        setUserActionBar()
        setIntentData()

        // Set RecyclerView
        val mAdapter = GroupAdapter(this)
        topicRecycler.layoutManager = LinearLayoutManager(this)
        topicRecycler.adapter = mAdapter

        // GetViewModel and observe topics data
        val viewModel = ViewModelProviders.of(this).get(GroupAdminViewModel::class.java)
        viewModel.topicID.value = topicID
        viewModel.groups.observe(this, Observer { data ->
            generalProgressBar.visibility = View.GONE
            if (data != null) {
                if (data.size > 0) {
                    hideNoDataText()
                    mAdapter.swapData(data)
                } else {
                    showNoDataText()
                }

            }
        })

    }

    private fun showNoDataText() {
        noDataLayout.visibility = View.VISIBLE
        noDataText.text = getString(R.string.group_page_no_data)
    }

    private fun hideNoDataText() {
        noDataLayout.visibility = View.GONE
    }

    override fun setIntentData() {
        val extra = intent
        if (extra != null &&
                extra.hasExtra(ExtraIntents.TOPIC_URL_IMAGE) &&
                extra.hasExtra(ExtraIntents.TOPIC_NAME) &&
                extra.hasExtra(ExtraIntents.TOPIC_ID) &&
                extra.hasExtra(ExtraIntents.CATEGORY_NAME)
                ) {

            // Set image/text/id of category
            previousText.text = extra.getStringExtra(ExtraIntents.CATEGORY_NAME)
            topicName = extra.getStringExtra(ExtraIntents.TOPIC_NAME)
            title = topicName
            topicID = extra.getStringExtra(ExtraIntents.TOPIC_ID)

            Glide.with(this).load(extra.getStringExtra(ExtraIntents.TOPIC_URL_IMAGE)).into(ivCategory)

            mTopicUrlImage = extra.getStringExtra(ExtraIntents.TOPIC_URL_IMAGE)

        }
    }


    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun setUserActionBar() {
        // Set actionBar
        setSupportActionBar(specialToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }

    override fun onClick(data: Group) {
        val intent = Intent(this, ExerciseActivity::class.java)
        intent.putExtra(ExtraIntents.GROUP_NAME, data.description)
        intent.putExtra(ExtraIntents.GROUP_ID, data.groupKey)
        intent.putExtra(ExtraIntents.TOPIC_NAME, topicName)
        intent.putExtra(ExtraIntents.TOPIC_URL_IMAGE, mTopicUrlImage)
        startActivity(intent)
    }
}