package cz.uhk.pavolpodstreleny.schoolproject.screens.tasks

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.util.Log
import android.view.MenuItem
import android.view.View
import com.bumptech.glide.Glide
import com.google.firebase.auth.FirebaseAuth
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Badge
import cz.uhk.pavolpodstreleny.schoolproject.model.Task
import cz.uhk.pavolpodstreleny.schoolproject.model.TaskOption
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.task.TaskAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.screens.result.ResultPage
import cz.uhk.pavolpodstreleny.schoolproject.utils.BadgeCalculator
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import kotlinx.android.synthetic.main.activity_task.*
import kotlinx.android.synthetic.main.view_app_bar.*
import kotlinx.android.synthetic.main.view_no_network_connectino_small.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch

/**
 * Created by pavol.podstreleny on 10.02.2018.
 */
class TaskActivity : AppCompatActivity(), TaskCancelDialog.OnDialogClick, INetworkChecker {

    private val LOG = "TASK_ACTIVITY"

    private var actualUserScore: Double? = null

    private var importantBadge: Badge? = null

    private var mUserBadges: List<String>? = null
    private var mAllBadges: List<Badge>? = null

    private var exerciseID: String? = null
    private var mTotalTasks: Int? = 0
    private var mTasks: ArrayList<Task> = ArrayList()
    private var mTaskOptions: ArrayList<TaskOption> = ArrayList()
    private var mAdapter: ChooserAdapter? = null
    private var mNextTask: Boolean = false
    private var taskPosition: Int = 0
    private var totalScore: Double = 0.0
    private var numberOfTries: Int = 0
    private var numberOfTriesManagable: Int = 1

    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_task)
        setIntentData()

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        // Get actual user
        val userID = FirebaseAuth.getInstance().currentUser?.uid

        //Check if user is logged in if not finish activity and show message in Exercise page :)
        if (userID == null) {
            stopActivity()
        }

        // Set actionBar
        setSupportActionBar(toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        mAdapter = ChooserAdapter()
        taskChoices.adapter = mAdapter
        taskChoices.layoutManager = LinearLayoutManager(this)

        val viewModel = ViewModelProviders.of(this).get(TaskAdminViewModel::class.java)
        viewModel.userID.value = userID
        viewModel.exerciseID.value = exerciseID
        viewModel.tasks.observe(this, Observer { data ->
            if (data != null) {
                if (data.size > 0) {
                    startProcess(data)
                    viewModel.numberOfTries.value = exerciseID + "/" + mTasks.get(taskPosition).taskID
                    noDataTaks.visibility = View.GONE
                    mainTaskViewPort.visibility = View.VISIBLE
                    bottomChecker.visibility = View.VISIBLE
                    hr.visibility = View.VISIBLE

                } else {
                    noDataTaks.visibility = View.VISIBLE
                    mainTaskViewPort.visibility = View.GONE
                    bottomChecker.visibility = View.GONE
                    hr.visibility = View.GONE
                }
            }
        })

        viewModel.number.observe(this, Observer { data ->
            if (data != null) {
                numberOfTries = data
            }

        })

        viewModel.userBadges.observe(this, Observer { data ->
            if (data != null) {
                mUserBadges = data
                if (mAllBadges != null) {
                    importantBadge = BadgeCalculator.calculateListBadge(mUserBadges!!, mAllBadges!!)
                }

            }
        })


        viewModel.getAllBadges().observe(this, Observer { data ->
            if (data != null) {
                mAllBadges = data
                if (mUserBadges != null) {
                    importantBadge = BadgeCalculator.calculateListBadge(mUserBadges!!, mAllBadges!!)
                }
            }
        })



        returnButton.setOnClickListener {
            finish()
        }

        viewModel.getUserPoints(userID!!).observe(this, Observer { data ->
            if (data != null) {
                actualUserScore = data

            }
        })



        dismissButton.setOnClickListener {
            dismissMessage()
        }

        checkNext.setOnClickListener {
            if (!NetworkUtil.isNetworkAvailable(this)) {
                Snacker.show(mainLayout, getString(R.string.task_no_internet_check))
                return@setOnClickListener
            }
            if (mNextTask) {
                showNextTaskOrFinalPage(viewModel)
            } else {

                val actualQuestionOptions = mAdapter?.getData()
                if (actualQuestionOptions != null) {

                    var answer = true

                    //Check if arrays are equal
                    for (x in 0..actualQuestionOptions.size - 1) {
                        if (actualQuestionOptions.get(x).isChecked != mTaskOptions.get(x).isChecked) {
                            answer = false
                        }
                    }

                    if (answer) {
                        if (actualUserScore != null) {
                            launch(UI) {

                                val specialValue = numberOfTries + 1
                                totalScore += 10 / (numberOfTries + numberOfTriesManagable)
                                val score = actualUserScore!! + 10 / (numberOfTries + numberOfTriesManagable)
                                val message = viewModel.setPoints(score, userID, exerciseID, mTasks.get(taskPosition).taskID, specialValue)

                                if (message.isSuccesfull) {
                                    showMessage(getString(R.string.task_page_correct_heading), getString(R.string.task_page_correct_text_body))
                                    if (importantBadge != null) {
                                        if (score >= importantBadge!!.points) {
                                            val result = viewModel.addBadge(userID, importantBadge!!.badgeName)
                                            if (result.isSuccesfull) {
                                                showMessage(getString(R.string.task_page_correct_heading), getString(R.string.task_page_correct_text_badge_body), badge = true, url = importantBadge!!.photoURL)
                                                viewModel.userID.value = userID
                                            }
                                        }
                                    }

                                    afterCorrectLayout.visibility = View.VISIBLE
                                    numberOfTries = 0
                                    numberOfTriesManagable = 1
                                } else {

                                }
                            }
                        }


                    } else {
                        showMessage(getString(R.string.task_page_wrong_heading), getString(R.string.task_page_wrong_text_body), false)
                        numberOfTriesManagable++
                    }
                }
            }
        }

    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == RESULT_OK) {
            finish()
        } else {
            Log.e(LOG, "Problem with getting right ")
        }
    }

    private fun showNextTaskOrFinalPage(viewModel: TaskAdminViewModel) {
        afterCorrectLayout.visibility = View.GONE
        if (mTotalTasks != null && mTotalTasks!! == ++taskPosition && mNextTask) {
            //Fire Final activity
            val intent = Intent(this, ResultPage::class.java)
            intent.putExtra(ExtraIntents.EXTRA_RESULT, totalScore)
            startActivityForResult(intent, ExtraIntents.RESULT_CODE)
        } else {
            dismissMessage()
            changeButtonText(getString(R.string.global_button_check_answer))
            viewModel.numberOfTries.value = exerciseID + "/" + mTasks.get(taskPosition).taskID
            mNextTask = false
            showTask(taskPosition)
        }
    }

    private fun stopActivity() {
        val intent = Intent()
        intent.putExtra(ExtraIntents.USER_MESSAGE, getString(R.string.task_page_no_registered))
        setResult(Activity.RESULT_OK, intent)
        finish()
    }

    private fun dismissMessage() {
        messageLayout.visibility = View.GONE
    }

    private fun showMessage(heading: String, text: String, answer: Boolean = true, badge: Boolean = false, url: String? = null) {
        if (badge) {
            badgeImage.visibility = View.VISIBLE
            Glide.with(this).load(url).into(badgeImage)
        } else {
            badgeImage.visibility = View.GONE
        }
        messageLayout.visibility = View.VISIBLE
        messageHeading.text = heading
        messageText.text = text
        if (answer) {
            messageColor.setBackgroundResource(R.color.colorGreen)
            changeButtonText(getString(R.string.global_button_next_task))
            mNextTask = true
        } else {
            messageColor.setBackgroundResource(R.color.colorRed)
        }
    }

    private fun changeButtonText(text: String) {
        checkNext.text = text
    }

    private fun startProcess(data: ArrayList<Task>) {
        mTasks = data
        showTask(taskPosition)
    }


    private fun showTask(taskWidthPosition: Int) {

        mTotalTasks = mTasks.size
        numberOfTasks.text = mTotalTasks.toString()
        numberOfTasksCompleted.text = (taskPosition + 1).toString()


        val task = mTasks.get(taskWidthPosition)
        taskText.text = task.description

        val listOfTaskOptions = arrayListOf<TaskOption>()
        mTaskOptions.clear()
        task.choices.forEach { t ->
            listOfTaskOptions.add(TaskOption(t.key, false))
            mTaskOptions.add(TaskOption(t.key, t.value))
        }

        mAdapter?.swapData(listOfTaskOptions)

    }


    private fun setIntentData() {
        val extra = intent
        if (extra != null && extra.hasExtra(ExtraIntents.EXERCISE_ID) && extra.hasExtra(ExtraIntents.EXERCISE_NAME)) {
            exerciseID = extra.getStringExtra(ExtraIntents.EXERCISE_ID)
            title = extra.getStringExtra(ExtraIntents.EXERCISE_NAME)
        }
    }

    private fun showDialog() {
        TaskCancelDialog().show(fragmentManager, "TASK_CANCEL_DIALOG")
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                showDialog()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        showDialog()
    }

    override fun onClickBackAccepted() {
        finish()
    }
}