package cz.uhk.pavolpodstreleny.schoolproject.screens.topics

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import com.bumptech.glide.Glide
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.UserActivities
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Topic
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionObserver
import cz.uhk.pavolpodstreleny.schoolproject.receivers.ConnectionReceiver
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.INetworkChecker
import cz.uhk.pavolpodstreleny.schoolproject.screens.admin.topic.TopicAdminViewModel
import cz.uhk.pavolpodstreleny.schoolproject.screens.groups.GroupActivity
import kotlinx.android.synthetic.main.activity_topic.*
import kotlinx.android.synthetic.main.view_general_progressbar.*
import kotlinx.android.synthetic.main.view_no_network_connectino_small.*


/**
 * Created by pavol.podstreleny on 09.02.2018.
 */
class TopicActivity : AppCompatActivity(), TopicAdapter.OnClickTopicListener, UserActivities, INetworkChecker {

    private var categoryID: String? = null
    private var categoryName: String? = null
    override var connectionObserver: ConnectionObserver? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_topic)

        connectionObserver = ConnectionObserver(ConnectionReceiver(noInternet), this, this)

        setIntentData()
        setUserActionBar()


        // Set RecyclerView
        val mAdapter = TopicAdapter(this, this)
        topicRecycler.layoutManager = LinearLayoutManager(this)
        topicRecycler.adapter = mAdapter

        // GetViewModel and observe topics data
        val viewModel = ViewModelProviders.of(this).get(TopicAdminViewModel::class.java)
        viewModel.topicID.value = categoryID
        viewModel.topics.observe(this, Observer { data ->
            generalProgressBar.visibility = View.GONE
            if (data != null) {
                if (data.size > 0) {
                    hideNoDataText()
                    mAdapter.swapData(data)
                } else {
                    showNoDataText()
                }

            }
        })


    }

    override fun onResume() {
        super.onResume()
        connectionObserver?.start()
    }

    private fun showNoDataText() {
        noDataLayout.visibility = View.VISIBLE
        noDataText.text = getString(R.string.topic_page_no_data)
    }

    private fun hideNoDataText() {
        noDataLayout.visibility = View.GONE
    }

    override fun setIntentData() {
        val extra = intent
        if (extra != null &&
                extra.hasExtra(ExtraIntents.CATEGORY_URL_IMAGE) &&
                extra.hasExtra(ExtraIntents.CATEGORY_NAME) &&
                extra.hasExtra(ExtraIntents.CATEGORY_ID)) {

            // Set image/text/id of category
            categoryName = extra.getStringExtra(ExtraIntents.CATEGORY_NAME)
            title = categoryName
            Glide.with(this).load(extra.getStringExtra(ExtraIntents.CATEGORY_URL_IMAGE)).into(ivCategory)
            categoryID = extra.getStringExtra(ExtraIntents.CATEGORY_ID)

        }
    }


    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        when (item?.getItemId()) {
            android.R.id.home -> {
                finish()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onClick(data: Topic?) {
        if (data != null) {
            val intent = Intent(this, GroupActivity::class.java)
            intent.putExtra(ExtraIntents.TOPIC_URL_IMAGE, data.photoURL)
            intent.putExtra(ExtraIntents.TOPIC_NAME, data.description)
            intent.putExtra(ExtraIntents.TOPIC_ID, data.topicKey)
            intent.putExtra(ExtraIntents.CATEGORY_NAME, categoryName)
            startActivity(intent)
        }
    }

    override fun setUserActionBar() {
        // Set actionBar
        setSupportActionBar(specialToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }
}