package cz.uhk.pavolpodstreleny.schoolproject.screens.user_profile

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.os.Bundle
import android.support.design.widget.TextInputEditText
import android.support.v4.app.Fragment
import android.support.v7.app.AlertDialog
import android.support.v7.widget.GridLayoutManager
import android.util.Log
import android.view.*
import com.google.firebase.auth.FirebaseAuth
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.ExtraIntents
import cz.uhk.pavolpodstreleny.schoolproject.model.Badge
import cz.uhk.pavolpodstreleny.schoolproject.model.UserProfile
import cz.uhk.pavolpodstreleny.schoolproject.utils.Snacker
import cz.uhk.pavolpodstreleny.schoolproject.utils.NetworkUtil
import kotlinx.android.synthetic.main.fragment_profile.*
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.launch
import java.text.SimpleDateFormat
import java.util.*


/**
 * Created by pavol.podstreleny on 22.12.2017.
 */
class UserProfileFragment : Fragment() {

    private val ZERO_TASKS_DONE = 0
    private val MIN_LENGHT_OF_USER_NAME = 3
    private val DATE_PATTERN = "dd.MM.yyyy"
    private val TAG = "UserProfileFragment"

    private var mProfile: UserProfile? = null
    private var mBadgesAdapter: UserBadgesAdapter? = null
    private val numberOfColumns = 3

    private var mUserUID: String? = null
    private var userDetail = false


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_profile, container, false)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)


        if(arguments != null){
            mUserUID = arguments!!.getString(ExtraIntents.USER_ID)
            editUserName.visibility = View.GONE
            badges.visibility = View.GONE
            userDetail = true

        }else{
            mUserUID = FirebaseAuth.getInstance().currentUser?.uid
        }

        //Set up recycler-view
        mBadgesAdapter = UserBadgesAdapter(this.context)
        badgeRecyclerView.adapter = mBadgesAdapter
        badgeRecyclerView.layoutManager = GridLayoutManager(context,numberOfColumns)

        if (mUserUID != null) {

            //Get View model
            val viewModel = ViewModelProviders.of(this).get(UserProfileViewModel::class.java)

            if(!userDetail) {
                viewModel.userIDBadges.value = mUserUID
            }

            viewModel.getCurrentUser(mUserUID!!).observe(this, Observer { data ->
                updateUserProfileUI(data)
            })

            viewModel.badges.observe(this, Observer {
                data ->
                updateUserBadgesUI(data)
            })

            //Set edit listener
            editUserName.setOnClickListener {
                createAlertDialog(viewModel)
            }
        }
    }

    private fun createAlertDialog(userViewModel: UserProfileViewModel) {
        val dialogView = layoutInflater.inflate(R.layout.dialog_rename,null)
        val editText = dialogView.findViewById<TextInputEditText>(R.id.userName)

        if(mProfile != null) {
            editText.setText(mProfile!!.name)
        }

        val builder = AlertDialog.Builder(activity!!,R.style.AlertDialogCustom).setView(dialogView)
        builder.setMessage(R.string.userProfile_page_dialog_header)
                .setPositiveButton(R.string.global_button_change, {
                    _, _ ->
                    changeUserName(editText,userViewModel)
                })
                .setNegativeButton(R.string.global_button_cancel, { dialog, _ -> dialog.cancel()})
        builder.create().show()
    }

    private fun changeUserName(editText: TextInputEditText,userViewModel: UserProfileViewModel) {
        val newName = editText.text.toString()
        if(newName.length >= MIN_LENGHT_OF_USER_NAME){
            if(NetworkUtil.isNetworkAvailable(activity!!.applicationContext)){
                launch(UI) {
                    val message = userViewModel.changeUserName(newName.trim())
                    Snacker.show(layout,getString(message.messageResource))
                }
            }else{
                Snacker.show(layout,getString(R.string.userProfile_page_user_name_change_error))
            }

        }else{
            Snacker.show(layout,getString(R.string.userProfile_page_user_name_text_limit))
        }
    }


    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
        if(!userDetail) {
            inflater?.inflate(R.menu.logout_menu, menu)
        }
        super.onCreateOptionsMenu(menu,inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        return when(item?.itemId){
            R.id.logout -> logout()
            else ->  super.onOptionsItemSelected(item)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
    }

    private fun updateUserBadgesUI(badges: List<Badge>?){
        if(badges != null){
            mBadgesAdapter?.swapData(badges)
        }
    }

    private fun updateUserProfileUI(profile: UserProfile?) {
        if (profile != null) {
            showLayout()

            mProfile = profile

            userName?.text = profile.name
            score?.text = profile.score.toString()

            if(profile.lastTask > ZERO_TASKS_DONE){
                setActualDate(profile)
            }else{
                setNoActivityDate()
            }

            if(profile.tasks != null){
                numberOfTasksCompleted.text = profile.tasks.size.toString()
            }else{
                numberOfTasksCompleted.text = ZERO_TASKS_DONE.toString()
            }

        }else{
            Log.e(TAG,"Null data!!")
        }
    }

    private fun showLayout(){
        progressBarUserProfile.visibility = View.GONE
        layout.visibility = View.VISIBLE
    }

    private fun setNoActivityDate() {
        date?.text = getString(R.string.userProfile_page_no_activity)
    }

    private fun setActualDate(profile: UserProfile) {
        val dates = Date(profile.lastTask)
        val actualDate = SimpleDateFormat(DATE_PATTERN, Locale.ENGLISH).format(dates)
        date?.text = actualDate
    }

    private fun logout(): Boolean{
        FirebaseAuth.getInstance().signOut()
        return true
    }

}
